package com.cusc.nirvana.user.rnr.common.dto;

import lombok.Data;

import java.io.*;

/**
 * Description: 实名消息队列DTO
 * <br />
 * CreateDate 2022-03-08 13:47
 *
 * @author yuy336
 **/
@Data
public class RnrMqMessageDTO<T> implements Serializable {

    /**
     * 投递的消息内容
     */
    private T data;

    /**
     * 延迟的总时间，秒
     */
    private Integer delayTime;

    /**
     * 剩余延迟的时间，秒。初始化与delayTime一致
     */
    private Integer surplusTime;

    /**
     * 当前投递的次数,第一次投递为1
     */
    private Integer count;

    /**
     * 是否是延迟消息
     */
    private boolean delayMsg;

    /**
     * 创建延迟消息
     *
     * @param delayTime
     * @param data
     * @param <T>
     * @return
     */
    public static <T> RnrMqMessageDTO<T> createDelayMsg(String delayTime, T data) {
        return createDelayMsg(Integer.parseInt(delayTime), data);
    }

    /**
     * 创建延迟消息
     *
     * @param delayTime
     * @param data
     * @param <T>
     * @return
     */
    public static <T> RnrMqMessageDTO<T> createDelayMsg(int delayTime, T data) {
        RnrMqMessageDTO<T> ret = new RnrMqMessageDTO<>();
        ret.setDelayMsg(true);
        ret.setData(data);
        ret.setCount(1);
        ret.setDelayTime(delayTime);
        ret.setSurplusTime(ret.getDelayTime());
        return ret;
    }

    /**
     * 创建正常消息
     *
     * @param data
     * @param <T>
     * @return
     */
    public static <T> RnrMqMessageDTO<T> createMsg(T data) {
        RnrMqMessageDTO<T> ret = new RnrMqMessageDTO<>();
        ret.setDelayMsg(false);
        ret.setData(data);
        ret.setCount(1);
        ret.setDelayTime(0);
        ret.setSurplusTime(0);
        return ret;
    }
}
