package com.cusc.nirvana.user.rnr.mg.controller;

import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrCompanyInfoDTO;
import com.cusc.nirvana.user.rnr.mg.service.IMgRnrCompanyInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * 实名公司信息表(MgRnrCompanyInfo)表控制层
 *
 * @author yuy336
 * @since 2022-02-10 18:30:29
 */
@RestController
@RequestMapping("/mgRnrCompanyInfo")
public class MgRnrCompanyInfoController {
    /**
     * 服务对象
     */
    @Autowired
    private IMgRnrCompanyInfoService mgRnrCompanyInfoService;

    /**
     * 通过主键查询单条数据
     *
     * @param bean 实体
     * @return 单条数据
     */
    @PostMapping("/getByUuid")
    public Response<MgRnrCompanyInfoDTO> getByUuid(@RequestBody MgRnrCompanyInfoDTO bean) {
        return Response.createSuccess(mgRnrCompanyInfoService.getByUuid(bean));
    }

    /**
     * 通过rnrid查询单条数据
     *
     * @param bean 实体
     * @return 单条数据
     */
    @PostMapping("/getByRnrid")
    public Response<MgRnrCompanyInfoDTO> getByRnrid(@RequestBody MgRnrCompanyInfoDTO bean) {
        return Response.createSuccess(mgRnrCompanyInfoService.getByRnrid(bean));
    }


    /**
     * 通过主键查询单条数据
     *
     * @param bean 实体
     * @return 单条数据
     */
    @PostMapping("/queryByList")
    public Response<List<MgRnrCompanyInfoDTO>> queryByList(@RequestBody MgRnrCompanyInfoDTO bean) {
        return Response.createSuccess(mgRnrCompanyInfoService.queryByList(bean));
    }

    /**
     * 新增数据
     *
     * @param bean 实体
     * @return 新增结果
     */
    @PostMapping("/add")
    public Response add(@RequestBody MgRnrCompanyInfoDTO bean) {
        return Response.createSuccess(mgRnrCompanyInfoService.insert(bean));
    }

    /**
     * 编辑数据
     *
     * @param bean 实体
     * @return 编辑结果
     */
    @PostMapping("/update")
    public Response update(@RequestBody MgRnrCompanyInfoDTO bean) {
        return Response.createSuccess(mgRnrCompanyInfoService.update(bean));
    }

    /**
     * 删除数据
     *
     * @param bean 实体
     * @return 删除是否成功
     */
    @PostMapping("/deleteById")
    public Response<Boolean> deleteById(@RequestBody MgRnrCompanyInfoDTO bean) {
        return Response.createSuccess(mgRnrCompanyInfoService.deleteById(bean));
    }

    /**
     * 根据公司名称查询rnrid
     */
    @PostMapping("/findRnrIdsByCompanyName")
    public Response<Boolean> findRnrIdsByCompanyName(@RequestBody MgRnrCompanyInfoDTO bean) {
        return Response.createSuccess(mgRnrCompanyInfoService.findRnrIdsByCompanyName(bean));
    }

}

