package com.cusc.nirvana.user.rnr.mg.controller;

import com.cusc.nirvana.common.result.PageResult;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.rnr.mg.dto.*;
import com.cusc.nirvana.user.rnr.mg.service.IOrgSimRelService;
import com.cusc.nirvana.user.rnr.mg.validation.group.Delete;
import com.cusc.nirvana.user.rnr.mg.validation.group.Insert;
import com.cusc.nirvana.user.rnr.mg.validation.group.Update;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * 组织对应SIM卡关系controller
 */
@RestController
@RequestMapping("/orgSimRel")
public class OrgSimRelController {

    @Autowired
    private IOrgSimRelService orgSimRelService;

    /**
     * 新增组织与SIM卡关系
     * @param orgSimRelDTO
     * @return
     */
    @PostMapping("add")
    public Response<OrgSimRelDTO> add(@RequestBody @Validated(Insert.class) OrgSimRelDTO orgSimRelDTO) {
        orgSimRelDTO = orgSimRelService.insert(orgSimRelDTO);
        return Response.createSuccess(orgSimRelDTO);
    }

    @PostMapping("addBatch")
    public Response<?> addBatch(@RequestBody OrgSimRelBatchDTO orgSimRelBatchDto) {
        Integer cnt = orgSimRelService.insertBatch(orgSimRelBatchDto.getOrgSimRels());
        return Response.createSuccess(cnt);
    }

    /**
     * 修改组织与SIM卡关系
     * @param orgSimRelDTO
     * @return
     */
    @PostMapping("update")
    public Response<OrgSimRelDTO> update(@RequestBody @Validated(Update.class) OrgSimRelDTO orgSimRelDTO) {
        orgSimRelDTO = orgSimRelService.update(orgSimRelDTO);
        return Response.createSuccess(orgSimRelDTO);
    }

    /**
     * 删除组织与卡关系
     * @param orgSimRelDTO
     * @return
     */
    @PostMapping("delete")
    public Response delete(@RequestBody @Validated(Delete.class) OrgSimRelDTO orgSimRelDTO) {
        boolean flag = orgSimRelService.deleteById(orgSimRelDTO);
        return Response.createSuccess(flag);
    }

    /**
     * 修改sim卡与vin码的绑定关系
     * @param simRelBindStatusDTO
     * @return
     */
    @PostMapping("updateBindStatus")
    public Response updateStatus(@RequestBody @Validated OrgSimRelBindStatusDTO simRelBindStatusDTO) {
        boolean flag = orgSimRelService.updateBindStatus(simRelBindStatusDTO.getIccid(), simRelBindStatusDTO.getVin(), simRelBindStatusDTO.getBindStatus());
        return Response.createSuccess(flag);
    }

    /**
     * 修改sim卡与vin码的绑定关系
     * @return
     */
    @PostMapping("updateBindIccid")
    public Response updateBindIccid(@RequestBody @Validated List<String> iccid) {
        boolean flag = orgSimRelService.updateBindIccid(iccid);
        return Response.createSuccess(flag);
    }

    /**
     * 查询指定数据
     * @param uuid
     * @return
     */
    @GetMapping("getByUuid")
    public Response<OrgSimRelDTO> getByUuid(String uuid) {
        OrgSimRelDTO orgSimRelDTO = orgSimRelService.getByUuid(uuid);
        return Response.createSuccess(orgSimRelDTO);
    }

    /**
     * 根据Iccid查询卡信息
     * @param iccid
     * @return
     */
    @GetMapping("getByIccid")
    public Response<OrgSimRelDTO> getByIccid(String iccid) {
        OrgSimRelDTO orgSimRelDTO = orgSimRelService.getByIccid(iccid);
        return Response.createSuccess(orgSimRelDTO);
    }

    /**
     * 根据vin码查询车卡关系列表
     * @param queryDTO
     * @return
     */
    @PostMapping("queryByVin")
    public Response<List<OrgSimRelDTO>> queryByVin(@RequestBody @Validated OrgSimRelVinQueryDTO queryDTO) {
        List<OrgSimRelDTO> list = orgSimRelService.queryByVin(queryDTO.getVin());
        return Response.createSuccess(list);
    }

    /**
     * 分页查询组织与卡关系
     * @param orgSimRelQueryDTO
     * @return
     */
    @PostMapping("queryByPage")
    @ApiOperation(value = "分页查询", notes = "分页查询")
    public Response<PageResult<OrgSimRelDTO>> queryByPage(@RequestBody @Validated OrgSimRelQueryDTO orgSimRelQueryDTO) {
        PageResult<OrgSimRelDTO> historyDtos = orgSimRelService.queryByPage(orgSimRelQueryDTO);
        return Response.createSuccess(historyDtos);
    }

    /**
     * 查询多个iccid详情
     * @param requestDTO
     * @return
     */
    @PostMapping("queryByList")
    @ApiOperation(value = "查询多个iccid详情")
    Response<List<OrgSimRelDTO>> queryByList(@RequestBody @Validated IccIdListRequestDTO requestDTO) {
        return Response.createSuccess(orgSimRelService.queryByList(requestDTO.getIccids()));
    }

    @PostMapping("updateVin")
    @ApiOperation(value = "更新卡信息")
    Response updateVin(@RequestBody @Validated List<MgRnrCardInfoDTO> mgRnrCardInfoDTOList) {
        orgSimRelService.updateVin(mgRnrCardInfoDTOList);
        return Response.createSuccess();
    }

    @PostMapping("queryInfoByIccidAndStatus")
    @ApiOperation(value = "根据iccid和卡状态去查询")
    public Response<List<OrgSimRelDTO>> queryInfoByIccidAndStatus(@RequestBody @Validated IccIdListRequestDTO requestDTO) {
        return Response.createSuccess(orgSimRelService.queryInfoByIccidAndStatus(requestDTO));
    }

    @PostMapping("updateStatusByIccid")
    @ApiOperation(value = "批量更新状态")
    public Response<List<OrgSimRelDTO>> updateStatusByIccid(@RequestBody @Validated IccIdListRequestDTO requestDTO) {
        orgSimRelService.updateStatusByIccid(requestDTO);
        return Response.createSuccess();
    }

}
