package com.cusc.nirvana.user.rnr.mg.controller;

import com.cusc.nirvana.common.result.PageResult;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.exception.CuscUserException;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrCardInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.RnrOrderDTO;
import com.cusc.nirvana.user.rnr.mg.service.IMgRnrCardInfoService;
import com.cusc.nirvana.user.rnr.mg.service.IMgRnrInfoService;
import com.cusc.nirvana.user.rnr.mg.service.IRnrOrderService;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;

/**
 * 实名工单信息(MgRnrOrder)表控制层
 *
 * @author yuy336
 * @since 2022-01-26 13:49:52
 */
@RestController
@RequestMapping("/mgRnrOrder")
public class RnrOrderController {
    /**
     * 服务对象
     */
    @Autowired
    private IRnrOrderService mgRnrOrderService;

    @Autowired
    private IMgRnrInfoService mgRnrInfoService;

    @Autowired
    private IMgRnrCardInfoService cardInfoService;

    /**
     * 分页查询
     *
     * @param bean 筛选条件
     * @return 查询结果
     */
    @PostMapping("/queryByPage")
    @ApiOperation(value = "分页查询", notes = "分页查询")
    public Response<PageResult<RnrOrderDTO>> queryByPage(@RequestBody RnrOrderDTO bean) {
        return Response.createSuccess(mgRnrOrderService.queryByPage(bean));
    }

    /**
     * 通过主键查询单条数据
     *
     * @param bean 实体
     * @return 单条数据
     */
   @PostMapping("/getByUuid")
    public Response<RnrOrderDTO> getByUuid(@RequestBody RnrOrderDTO bean) {
        return Response.createSuccess(mgRnrOrderService.getByUuid(bean));
    }
    
     /**
     * 通过主键查询单条数据
     *
     * @param bean 实体
     * @return 单条数据
     */
   @PostMapping("/queryByList")
    public Response<List<RnrOrderDTO>> queryByList(@RequestBody RnrOrderDTO bean) {
        return Response.createSuccess(mgRnrOrderService.queryByList(bean));
    }

    /**
     * 新增数据
     *
     * @param bean 实体
     * @return 新增结果
     */
    @PostMapping("/add")
    public Response add(@RequestBody RnrOrderDTO bean) {
       return Response.createSuccess(mgRnrOrderService.insert(bean));
    }

    /**
     * 编辑数据
     *
     * @param bean 实体
     * @return 编辑结果
     */
    @PostMapping("/update")
    public Response update(@RequestBody RnrOrderDTO bean) {
        return Response.createSuccess(mgRnrOrderService.update(bean));
    }

    /**
     * 删除数据
     *
     * @param bean 实体
     * @return 删除是否成功
     */
    @PostMapping("/deleteById")
    public Response<Boolean> deleteById(@RequestBody RnrOrderDTO bean) {
        return Response.createSuccess(mgRnrOrderService.deleteById(bean));
    }


    @PostMapping("/startWorkOrder")
	public Response startWorkOrder(@RequestBody RnrOrderDTO dto){
        //查询工单信息
        dto = mgRnrOrderService.getByUuid(dto);

        //查询实名信息
        MgRnrInfoDTO mgRnrInfoDTO = new MgRnrInfoDTO();
        mgRnrInfoDTO.setUuid(dto.getRnrId());
        mgRnrInfoDTO = mgRnrInfoService.getByUuid(mgRnrInfoDTO);
        if(mgRnrInfoDTO == null){
            throw  new CuscUserException("","查询工单对应业务失败");
        }

        //查询卡信息
        MgRnrCardInfoDTO mgRnrCardInfoDTO = new MgRnrCardInfoDTO();
        mgRnrCardInfoDTO.setOrderId(dto.getUuid());
        mgRnrCardInfoDTO.setCurrPage(1);
        mgRnrCardInfoDTO.setPageSize(5);
        PageResult<MgRnrCardInfoDTO> page = cardInfoService.queryByPage(mgRnrCardInfoDTO);

        mgRnrOrderService.sendOrderSystem(dto,mgRnrInfoDTO,page.getList());
    	return Response.createSuccess();
	}
    @PostMapping("/updateOrderStatus")
    public Response updateOrderStatus(@RequestBody RnrOrderDTO dto){
        List<String> orderIds = new ArrayList<>();
        orderIds.add(dto.getUuid());
        mgRnrOrderService.updateOrderStatus(orderIds,dto.getOrderStatus());
        return Response.createSuccess("更新工单成功");
    }
}

