package com.cusc.nirvana.user.rnr.mg.controller;

import com.cusc.nirvana.common.result.PageResult;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.rnr.mg.dto.RnrOrderDTO;
import com.cusc.nirvana.user.rnr.mg.dto.SimFileHistoryDTO;
import com.cusc.nirvana.user.rnr.mg.dto.SimFileHistoryQueryDTO;
import com.cusc.nirvana.user.rnr.mg.service.ISimFileHistoryService;
import com.cusc.nirvana.user.rnr.mg.service.impl.RnrOrderServiceImpl;
import com.cusc.nirvana.user.rnr.mg.validation.group.Delete;
import com.cusc.nirvana.user.rnr.mg.validation.group.Insert;
import com.cusc.nirvana.user.rnr.mg.validation.group.Update;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.Date;
import java.util.List;

/**
 * Sim卡导入历史对外controller
 */
@RestController
@RequestMapping("/simFileHistory")
public class SimFileHistoryController {

    @Autowired
    private ISimFileHistoryService fileHistoryService;
    @Autowired
    private RnrOrderServiceImpl rnrOrderService;

    /**
     * 新增SIM卡导入历史信息
     * @param historyDTO
     * @return
     */
    @PostMapping("add")
    public Response<SimFileHistoryDTO> add(@RequestBody @Validated(Insert.class) SimFileHistoryDTO historyDTO) {
        historyDTO.setCreateTime(new Date());
        historyDTO = fileHistoryService.insert(historyDTO);
        return Response.createSuccess(historyDTO);
    }

    /**
     * 修改SIM卡导入历史信息
     * @param historyDTO
     * @return
     */
    @PostMapping("update")
    public Response<SimFileHistoryDTO> update(@RequestBody @Validated(Update.class) SimFileHistoryDTO historyDTO) {
        historyDTO = fileHistoryService.update(historyDTO);
        return Response.createSuccess(historyDTO);
    }

    /**
     * 删除SIM卡导入历史信息
     * @param historyDTO
     * @return
     */
    @PostMapping("delete")
    public Response delete(@RequestBody @Validated(Delete.class) SimFileHistoryDTO historyDTO) {
        boolean flag = fileHistoryService.deleteById(historyDTO.getOrgUuid(), historyDTO.getUuid());
        return Response.createSuccess(flag);
    }

    /**
     * 查询指定数据
     * @param uuid
     * @return
     */
    @GetMapping("getByUuid")
    public Response<SimFileHistoryDTO> getByUuid(String uuid) {
        SimFileHistoryDTO historyDTO = fileHistoryService.getByUuid(uuid);
        return Response.createSuccess(historyDTO);
    }

    /**
     * 分页查询历史数据
     * @param queryDTO
     * @return
     */
    @PostMapping("queryByPage")
    @ApiOperation(value = "分页查询", notes = "分页查询")
    public Response<PageResult<SimFileHistoryDTO>> queryByPage(@RequestBody @Validated SimFileHistoryQueryDTO queryDTO) {
        PageResult<SimFileHistoryDTO> historyDtos = fileHistoryService.queryByPage(queryDTO);
        return Response.createSuccess(historyDtos);
    }

}
