package com.cusc.nirvana.user.rnr.mg.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.cusc.nirvana.common.result.PageResult;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.rnr.mg.dao.entity.OrgSimRelPO;
import com.cusc.nirvana.user.rnr.mg.dto.IccIdListRequestDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrCardInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.OrgSimRelDTO;
import com.cusc.nirvana.user.rnr.mg.dto.OrgSimRelQueryDTO;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.List;

public interface IOrgSimRelService extends IService<OrgSimRelPO> {

    /**
     * 通过uuid查询单条数据
     * @param uuid
     * @return
     */
    OrgSimRelDTO getByUuid(String uuid);

    /**
     * 根据Iccid查询卡信息
     * @param iccid
     * @return
     */
    OrgSimRelDTO getByIccid(String iccid);

    /**
     * 查询分页列表
     * @param queryDTO
     * @return
     */
    PageResult<OrgSimRelDTO> queryByPage(OrgSimRelQueryDTO queryDTO);

    /**
     * 查询多个iccid详情
     * @param iccids
     * @return
     */
    List<OrgSimRelDTO> queryByList(List<String> iccids);

    /**
     * 根据vin查询车卡关系列表
     * @param vin
     * @return
     */
    List<OrgSimRelDTO> queryByVin(String vin);

    /**
     * 新增组织和SIM卡的关系
     * @param simRelDTO
     * @return
     */
    OrgSimRelDTO insert(OrgSimRelDTO simRelDTO);

    /**
     * 批量新增组织和SIM卡的关系
     * @param simRelDTOS
     * @return
     */
    Integer insertBatch(List<OrgSimRelDTO> simRelDTOS);

    /**
     * 修改组织和SIM卡的关系
     * @param simRelDTO
     * @return
     */
    OrgSimRelDTO update(OrgSimRelDTO simRelDTO);

    /**
     * 更新车卡关系的绑定状态，0:绑定，1：解绑
     * @param iccid
     * @param vin
     * @param bindStatus
     */
    boolean updateBindStatus(String iccid, String vin, Integer bindStatus);


    boolean updateBindIccid(List<String> iccid);


    /**
     * 删除指定数据
     * @param simRelDTO
     * @return
     */
    boolean deleteById(OrgSimRelDTO simRelDTO);

     /**
       * @author: jk
       * @description: 同步vin
       * @date: 2022/7/13 9:36
       * @version: 1.0
       */
     void updateVin(List<MgRnrCardInfoDTO> mgRnrCardInfoDTOList);


    /**
     * 查询符合要求的数据
     *
     * @param requestDTO
     * @return
     */
    List<OrgSimRelDTO> queryInfoByIccidAndStatus(IccIdListRequestDTO requestDTO);

    /**
     * 批量更新状态
     *
     * @param requestDTO
     * @return
     */
    boolean updateStatusByIccid(IccIdListRequestDTO requestDTO);
}
