package com.cusc.nirvana.user.rnr.mg.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cache.CacheFactory;
import com.cache.exception.CacheException;
import com.cusc.nirvana.common.loader.CollectionUtils;
import com.cusc.nirvana.common.result.PageResult;
import com.cusc.nirvana.rds.mybatis.PageHelper;
import com.cusc.nirvana.user.rnr.common.constants.RedisConstant;
import com.cusc.nirvana.user.rnr.mg.constants.CommonYesOrNoEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrBizzTypeEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrStatus;
import com.cusc.nirvana.user.rnr.mg.dao.MgCheckProgressDao;
import com.cusc.nirvana.user.rnr.mg.dto.MgCheckProgressDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgCheckStatisticsDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgCheckStatisticsQueryDTO;
import com.cusc.nirvana.user.rnr.mg.service.IMgCheckProgressService;
import com.cusc.nirvana.user.util.DateUtils;
import com.cusc.nirvana.user.util.crypt.CryptKeyUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @author hxin
 * @date 2022/5/18 19:15
 * @modify 2022-06-01
 */
@Service
@Slf4j
public class MgCheckProgressServiceImpl implements IMgCheckProgressService {

    @Resource
    private MgCheckProgressDao mgCheckProgressDao;

    @Resource
    private CacheFactory cacheFactory;

    private static final Integer MAX_NUMBER = 500;

    @Override
    public PageResult<MgCheckProgressDTO> queryByPage(MgCheckProgressDTO bean) {

        bean.setFullName(CryptKeyUtil.encryptToBase64(bean.getFullName()));
        bean.setPhone(CryptKeyUtil.encryptToBase64(bean.getPhone()));
        bean.setIotId(CryptKeyUtil.encryptToBase64(bean.getIotId()));
        bean.setIccid(CryptKeyUtil.encryptToBase64(bean.getIccid()));
        bean.setCompanyName(CryptKeyUtil.encryptToBase64(bean.getCompanyName()));
        Page<MgCheckProgressDTO> page = new Page<>(bean.getCurrPage(), bean.getPageSize());
        page.setRecords(mgCheckProgressDao.queryCheckProgressByPage(page, bean));
        return PageHelper.convert(page, MgCheckProgressDTO.class);

    }

    /**
     * @modify  加入租户和用户的所属车企的所有下属组织做筛选
     *          组织idList比较多
     *          考虑改成 for循环分批查询再累计
     *          redis key采用 租户编号+用户当前组织id+日期
     * @param queryDTO
     * @return
     */
    @Override
    public MgCheckStatisticsDTO queryCheckStatistics(MgCheckStatisticsQueryDTO queryDTO) {

        String cachedValue = this.getCachedStatisticsValue(queryDTO);
        if(StringUtils.isNotBlank(cachedValue)){
            return JSON.parseObject(cachedValue,MgCheckStatisticsDTO.class);
        }

        List<List<String>> subList = this.subList(queryDTO.getOrgIdList()); //orgIdList切分 分批查询

        //查询参数
        MgCheckStatisticsQueryDTO queryPeopleDTO = this.initQueryParam(CommonYesOrNoEnum.NO.getCode(),queryDTO.getTenantNo());
        MgCheckStatisticsQueryDTO queryCompanyDTO = this.initQueryParam(CommonYesOrNoEnum.YES.getCode(),queryDTO.getTenantNo());

        //车企的所有的实名数目 状态 1 2 3
        List<Integer> rnrStatusAllList = new ArrayList<>();
        rnrStatusAllList.add(RnrStatus.RNR.getCode());
        rnrStatusAllList.add(RnrStatus.RNR_FAIL.getCode());
        rnrStatusAllList.add(RnrStatus.UNBOUND.getCode());
        MgCheckStatisticsQueryDTO allParam = initVehicleQueryParam(rnrStatusAllList);
        allParam.setTenantNo(queryDTO.getTenantNo());

        //车企的成功的实名数目 状态1 3
        List<Integer> rnrStatusSuccessList = new ArrayList<>();
        rnrStatusSuccessList.add(RnrStatus.RNR.getCode());
        rnrStatusSuccessList.add(RnrStatus.UNBOUND.getCode());
        MgCheckStatisticsQueryDTO successParam = initVehicleQueryParam(rnrStatusSuccessList);
        successParam.setTenantNo(queryDTO.getTenantNo());


        Integer peopleRnrNumTotal = 0;
        Integer companyRnrNumTotal = 0;
        Integer allNumTotal = 0;
        Integer successNumTotal = 0;
        for(List<String> orgIdList : subList){

            queryPeopleDTO.setOrgIdList(orgIdList);
            Integer peopleRnrNum = mgCheckProgressDao.queryStatisticsCountByCondition(queryPeopleDTO);
            peopleRnrNumTotal += peopleRnrNum;

            queryCompanyDTO.setOrgIdList(orgIdList);
            Integer companyRnrNum = mgCheckProgressDao.queryStatisticsCountByCondition(queryCompanyDTO);
            companyRnrNumTotal += companyRnrNum;

            allParam.setOrgIdList(orgIdList);
            Integer allNum = mgCheckProgressDao.queryStatisticsCountByCondition(allParam);
            allNumTotal += allNum;

            successParam.setOrgIdList(orgIdList);
            Integer successNum = mgCheckProgressDao.queryStatisticsCountByCondition(successParam);
            successNumTotal += successNum;
        }

        String rate;
        if(allNumTotal == 0 || successNumTotal == 0){
            rate = "暂无车企数据";
        }else {
            rate = calculatePercent(successNumTotal,allNumTotal);
        }

        //车企处理平均时长
        String vehicleHandleTimeAvg = this.getVehicleHandleTimeAvg(queryDTO, subList);

        MgCheckStatisticsDTO result = new MgCheckStatisticsDTO();
        result.setPeopleRnrNum(peopleRnrNumTotal);
        result.setCompanyRnrNum(companyRnrNumTotal);
        result.setVehicleCompanyRnrSuccessRate(rate);
        result.setVehicleHandleTimeAvg(vehicleHandleTimeAvg);

        this.setCachedStatisticsValue(result,queryDTO);
        return result;
    }

    public String getCachedStatisticsValue(MgCheckStatisticsQueryDTO queryDTO){
        String key = RedisConstant.RNR_CARD_STATISTICS_KEY+queryDTO.getTenantNo()+":"+queryDTO.getUserOrgId()+initDate().substring(0,10);
        String valueStr = null;
        try {
            valueStr = cacheFactory.getStringService().getValue(key,String.class);
        } catch (CacheException e) {
            log.error("MgCheckProgressServiceImpl.getCachedCheckStatistics 获取缓存失败,key = {}", key, e);
        }
        return valueStr;
    }

    public void setCachedStatisticsValue(MgCheckStatisticsDTO result,MgCheckStatisticsQueryDTO queryDTO){
        String key = RedisConstant.RNR_CARD_STATISTICS_KEY+queryDTO.getTenantNo()+":"+queryDTO.getUserOrgId()+initDate().substring(0,10);
        try {
            cacheFactory.getExpireStringService().setExpireValue(key,JSON.toJSONString(result),60*60*24);
        } catch (CacheException e) {
            log.error("MgCheckProgressServiceImpl.setCachedValue 加入缓存失败,key = {}", key, e);

        }
    }

    /**
     * 车企处理平均时长
     * @return
     */
    private String getVehicleHandleTimeAvg(MgCheckStatisticsQueryDTO queryDTO,List<List<String>> subList){
        MgCheckStatisticsQueryDTO param = new MgCheckStatisticsQueryDTO();
        param.setDateTime(initDate());
        param.setTenantNo(queryDTO.getTenantNo());

        List<Integer> rnrBizzTypeList = new ArrayList<>();
        rnrBizzTypeList.add(RnrBizzTypeEnum.Bind.getCode());
        rnrBizzTypeList.add(RnrBizzTypeEnum.Unbound.getCode());
        rnrBizzTypeList.add(RnrBizzTypeEnum.ChangeBinding.getCode());
        rnrBizzTypeList.add(RnrBizzTypeEnum.INFO_CHANGE.getCode());
        rnrBizzTypeList.add(RnrBizzTypeEnum.REPEAT_BIND.getCode());
        param.setRnrBizzTypeList(rnrBizzTypeList);
        //param.setIsCompany(CommonYesOrNoEnum.YES.getCode());
        //param.setIsVehicleCompany(CommonYesOrNoEnum.YES.getCode());

        //分批查询
        List<Integer> all = new ArrayList<>();
        for(List<String> orgIdList : subList){
            param.setOrgIdList(orgIdList);
            List<Integer> list = mgCheckProgressDao.queryVehicleCompanyHandleTime(param);
            all.addAll(list);
        }
        List<Integer> collect = all.stream().filter(Objects::nonNull).collect(Collectors.toList());
        log.info("collect:"+collect);

        if(CollectionUtils.isEmpty(collect)){
            return "暂无平均处理时长";
        }
        //计算平均值
        Double average = collect.stream().mapToDouble(Number::doubleValue).average().getAsDouble();
        String s = secondToTime(average.longValue());
        log.info("average"+s);
        return s;
    }

    /**
     *  秒转为时分秒
     * @param
     * @return
     */
    private static String secondToTime(long second) {

        long days = second /86400;//转换天数
        second = second %86400;//剩余秒数
        long hours = second /3600;//转换小时数
        second = second %3600;//剩余秒数
        long minutes = second /60;//转换分钟
        second = second %60;//剩余秒数
        if (0 < days){
            return days +":"+hours+":"+minutes+":"+second;
        }else {
            return hours+":"+minutes+":"+second;
        }

    }

    /**
     * 计算切分次数
     */
    private  Integer countStep(Integer size) {
        return (size + MAX_NUMBER - 1) / MAX_NUMBER;
    }

    /**
     * 切分list
     * @param list
     * @return
     */
    private List<List<String>> subList(List<String> list){
        int limit = countStep(list.size());

        List<List<String>> mglist = new ArrayList<>();
        if(CollectionUtils.isEmpty(list)){
            return mglist;
        }

        Stream.iterate(0, n -> n + 1).limit(limit).forEach(i -> {
            mglist.add(list.stream().skip(i * MAX_NUMBER).limit(MAX_NUMBER).collect(Collectors.toList()));
        });

        return mglist;
    }

    /**
     * 自然人或企业
     * 实名查询参数
     * @modify  rnr_status改为1和3 失败不统计
     * @return
     */
    private MgCheckStatisticsQueryDTO initQueryParam(Integer isCompany,String tenantNo){
        MgCheckStatisticsQueryDTO queryDTO = new MgCheckStatisticsQueryDTO();
        queryDTO.setDateTime(initDate());
        queryDTO.setTenantNo(tenantNo);

        List<Integer> rnrBizzTypeList = new ArrayList<>();
        rnrBizzTypeList.add(RnrBizzTypeEnum.Bind.getCode());
        queryDTO.setRnrBizzTypeList(rnrBizzTypeList);

        List<Integer> rnrStatusList = new ArrayList<>();
        rnrStatusList.add(RnrStatus.RNR.getCode());
        //rnrStatusList.add(RnrStatus.RNR_FAIL.getCode());
        rnrStatusList.add(RnrStatus.UNBOUND.getCode());

        queryDTO.setRnrStatusList(rnrStatusList);
        queryDTO.setIsCompany(isCompany);
        return queryDTO;
    }

    /**
     * 车企实名成功率查询参数
     * @return
     */
    private MgCheckStatisticsQueryDTO initVehicleQueryParam(List<Integer> rnrStatusList){
        MgCheckStatisticsQueryDTO queryDTO = new MgCheckStatisticsQueryDTO();
        queryDTO.setDateTime(initDate());

        List<Integer> rnrBizzTypeList = new ArrayList<>();
        rnrBizzTypeList.add(RnrBizzTypeEnum.Bind.getCode());
        queryDTO.setRnrBizzTypeList(rnrBizzTypeList);

        queryDTO.setRnrStatusList(rnrStatusList);
        //queryDTO.setIsCompany(CommonYesOrNoEnum.YES.getCode());
        //queryDTO.setIsVehicleCompany(CommonYesOrNoEnum.YES.getCode());

        return queryDTO;
    }

    /**
     * 获取凌晨时间
     * @return
     */
    public static String initDate(){
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        return DateUtils.formatDatetime(calendar.getTime());
    }

    /**
     * a / b   计算百分比
     *
     */
    public static String calculatePercent(Integer a , Integer b){
        //保留两位 四舍五入
        String percent = new BigDecimal((double)a*100/b).setScale(0,BigDecimal.ROUND_HALF_UP)+"%";
        return percent;
    }

}
