package com.cusc.nirvana.user.rnr.mg.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.nacos.common.utils.CollectionUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cache.CacheFactory;
import com.cache.exception.CacheException;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.eiam.dto.OrganizationDTO;
import com.cusc.nirvana.user.rnr.common.constants.RedisConstant;
import com.cusc.nirvana.user.rnr.mg.constants.NoticeStatusEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrBizzTypeEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrMgMqConstant;
import com.cusc.nirvana.user.rnr.mg.constants.RnrOrderType;
import com.cusc.nirvana.user.rnr.mg.converter.MgRnrCardInfoConverter;
import com.cusc.nirvana.user.rnr.mg.dao.entity.MgRnrCardInfoPO;
import com.cusc.nirvana.user.rnr.mg.dto.MgCardNoticeDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrCardInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrTaskDTO;
import com.cusc.nirvana.user.rnr.mg.dto.RnrOrderDTO;
import com.cusc.nirvana.user.rnr.mg.service.*;
import com.cusc.nirvana.user.rnr.mg.util.RnrOrderUtil;
import com.cusc.nirvana.user.rnr.notice.kafka.KafkaProduct;
import com.cusc.nirvana.user.rnr.workorder.context.WorkOrderCallBackContext;
import com.cusc.nirvana.user.rnr.workorder.dto.WorkOrderReviewCallBackRequestDTO;
import com.cusc.nirvana.user.rnr.workorder.handler.WorkOrderCallBackHandler;
import com.cusc.nirvana.user.util.CuscStringUtils;
import com.cusc.nirvana.user.util.DateUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * 消息发送实现类
 *
 * @author yubo
 * @since 2022-04-16 14:53
 */
@Service
@Slf4j
@EnableScheduling
public class MgRnrCardNoticeServiceImpl implements IMgRnrCardNoticeService {

    @Autowired
    IMgRnrCardInfoService cardInfoService;

    @Autowired
    IRnrOrderService orderService;

    @Resource
    private CacheFactory cacheFactory;

    @Autowired
    private KafkaProduct kafkaProduct;

    @Value("${mg.rnr.mq.lock.expireTime:6000}")
    private int expireTime;

    @Autowired
    private IOrganService organService;

    @Resource
    private WorkOrderCallBackContext context;

    @Resource
    private IMgRnrTaskService mgRnrTaskService;

    @Scheduled(fixedDelayString = "${mg.rnr.mq.interval:300000}")
    @Override
    public void sendCardNotices() {
        log.info("MgRnrCardNoticeServiceImpl sendCardNotices start {}", DateUtils.currentTimeStr());
        Iterator<MgRnrCardInfoPO> it = findNoticeCards();
        while (it.hasNext()) {
            MgRnrCardInfoPO cardInfoPO = it.next();
            RnrOrderDTO rnrOrderDTO = new RnrOrderDTO();
            rnrOrderDTO.setUuid(cardInfoPO.getOrderId());
            rnrOrderDTO = orderService.getByUuid(rnrOrderDTO);

            //如果是换件，需要查询老卡iccid。用于T1上报
            if (RnrOrderType.TBOX_CHANGE.getCode() == rnrOrderDTO.getOrderType()) {
                MgRnrCardInfoDTO mgRnrCardInfoDTO = new MgRnrCardInfoDTO();
                mgRnrCardInfoDTO.setUuid(cardInfoPO.getOldCardId());
                MgRnrCardInfoDTO mgRnrCardInfo = cardInfoService.getByUuid(mgRnrCardInfoDTO);
                if (null != mgRnrCardInfo) {
                    cardInfoPO.setOldCardId(mgRnrCardInfo.getIccid());
                }
            }
            sendCardNotice(convertToNotice(cardInfoPO, RnrOrderUtil.orderTypeToNoticeType(rnrOrderDTO.getOrderType()),
                    rnrOrderDTO));        }
    }

    /**
     * 发送实名卡通知
     */
    @Async("mqExecutor")
    @Override
    public void sendCardNotice(List<MgCardNoticeDTO> dtos) {
        for (MgCardNoticeDTO dto : dtos) {
            sendCardNotice(dto);
        }
    }

    @Override
    public void sendCardNoticeByOrderId(String orderId) {
        RnrOrderDTO rnrOrderDTO = new RnrOrderDTO();
        rnrOrderDTO.setUuid(orderId);
        rnrOrderDTO = orderService.getByUuid(rnrOrderDTO);

        //查询车企信息
        String manufacturerOrgId;
        OrganizationDTO organ = new OrganizationDTO();
        organ.setTenantNo(rnrOrderDTO.getTenantNo());
        organ.setUuid(rnrOrderDTO.getOrgId());
        Response<OrganizationDTO> organResp = organService.getCarSubOrganByOrgId(organ);
        if (organResp != null && organResp.isSuccess() && organResp.getData() != null) {
            manufacturerOrgId = organResp.getData().getUuid();
        }else{
            log.warn("sendCardNoticeByOrderId 获取车企组织失败，{}", JSON.toJSONString(organResp));
            manufacturerOrgId = "0";
        }

        QueryWrapper<MgRnrCardInfoPO> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("order_id", orderId);
        List<MgRnrCardInfoPO> list = cardInfoService.list(queryWrapper);

        List<MgCardNoticeDTO> dtos = new ArrayList<>();
        for (MgRnrCardInfoPO cardInfo : list) {
            MgCardNoticeDTO mgCardNoticeDTO =
                    convertToNotice(cardInfo, RnrOrderUtil.orderTypeToNoticeType(rnrOrderDTO.getOrderType()),
                            rnrOrderDTO);
            if (mgCardNoticeDTO != null) {
                mgCardNoticeDTO.setManufacturerOrgId(manufacturerOrgId);
                dtos.add(mgCardNoticeDTO);
            }
        }
        sendCardNotice(dtos);
    }


    // ------------------ 私有方法区 --------------------

    /**
     * 发送卡操作消息
     *
     * @param dto
     */
    private void sendCardNotice(MgCardNoticeDTO dto) {
        String lockKey = RedisConstant.RNR_CARD_NOTICE_LOCK + dto.getOrderId() + ":" + dto.getIccid();

        try {
            Boolean lock = cacheFactory.getLockService().lock(lockKey, expireTime);
            if (!lock) {
                return;
            }

            kafkaProduct.sendMessage(RnrMgMqConstant.RNR_CARD_OPERATION_TOPIC, JSON.toJSONString(dto));

            log.info("发送实名kafka消息成功:{}", JSON.toJSONString(dto));

            MgRnrCardInfoDTO mgRnrCardInfoDTO = cardInfoService.getByOrderIdAndIccId(dto.getOrderId(), dto.getIccid
                    ());
            log.warn("mgRnrCardInfoDTO==={}", JSON.toJSON(mgRnrCardInfoDTO));
            if (mgRnrCardInfoDTO != null) {
                MgRnrCardInfoPO cardInfoPO = new MgRnrCardInfoPO();
                cardInfoPO.setId(mgRnrCardInfoDTO.getId());
                cardInfoPO.setNoticeStatus(NoticeStatusEnum.SUCCESS.getCode());
                cardInfoService.updateById(cardInfoPO);
                log.info("更新发送状态:{}", JSON.toJSONString(dto));
            }
        } catch (Exception e) {
            log.error("RetryUploadInfo failed, card notice: " + dto.getOrderId() + ":" + dto.getIccid(), e);
        } finally {
            try {
                cacheFactory.getLockService().unLock(lockKey);
            } catch (CacheException e) {
                log.error("Failed to unlock " + lockKey, e);
            }
        }
    }

    /**
     * 转成要发送的消息体
     *
     * @param cardInfo
     * @return
     */
    private MgCardNoticeDTO convertToNotice(MgRnrCardInfoPO cardInfo, RnrBizzTypeEnum rnrNoticeTypeEnum,
                                            RnrOrderDTO rnrOrderDTO) {
        MgCardNoticeDTO mqDTO = MgRnrCardInfoConverter.INSTANCE.poToNoticeDTO(cardInfo);
        mqDTO.setVin(cardInfo.getIotId());
        mqDTO.setRnrBizzType(rnrNoticeTypeEnum.getCode());
        mqDTO.setOrderId(cardInfo.getOrderId());
        mqDTO.setOldIccid(cardInfo.getOldCardId());
        mqDTO.setIccid(cardInfo.getIccid());
        mqDTO.setManufacturerOrgId(rnrOrderDTO.getOrgId());
        mqDTO.setResultCode(rnrOrderDTO.getOrderStatus());
        mqDTO.setTenantNo(rnrOrderDTO.getTenantNo());
        return mqDTO;
    }

    /**
     * 获取待发送卡消息
     */
    private Iterator<MgRnrCardInfoPO> findNoticeCards() {
        return new Iterator<MgRnrCardInfoPO>() {
            private Page<MgRnrCardInfoPO> cardInfoPOs;
            int currentPage = 1;
            int currentIndex = 0;

            @Override
            public boolean hasNext() {
                if (cardInfoPOs != null && currentIndex < cardInfoPOs.getRecords().size()) {
                    return true;
                }
                if (cardInfoPOs == null) {
                    cardInfoPOs = findNexPage(currentPage++);
                    currentIndex = 0;
                } else if (currentIndex >= cardInfoPOs.getRecords().size()) {
                    currentIndex = 0;
                    if (cardInfoPOs.hasNext()) {
                        cardInfoPOs = findNexPage(currentPage++);
                    } else {
                        cardInfoPOs = null;
                    }
                }
                return cardInfoPOs != null && !cardInfoPOs.getRecords().isEmpty();
            }

            @Override
            public MgRnrCardInfoPO next() {
                return cardInfoPOs.getRecords().get(currentIndex++);
            }
        };
    }

    private Page<MgRnrCardInfoPO> findNexPage(int page) {
        QueryWrapper<MgRnrCardInfoPO> queryWrapper = new QueryWrapper<>();
        //notice_status != 0 and notice_status != 2
        queryWrapper.eq("notice_status", NoticeStatusEnum.NEED.getCode());
        queryWrapper.orderByAsc("create_time");
        return cardInfoService.page(new Page<>(page, 100), queryWrapper);
    }

    @Async("asyncOrderExecutor")
    @Override
    public void asyncOrderExecutor(WorkOrderReviewCallBackRequestDTO callBackRequestDTO) {
        WorkOrderCallBackHandler workBackHandler = context.getWorkBackHandler(callBackRequestDTO.getWorkOrderKey());
        MgRnrTaskDTO mgRnrTaskDTO = new MgRnrTaskDTO();
        mgRnrTaskDTO.setUuid(CuscStringUtils.generateUuid());
        mgRnrTaskDTO.setBizKey(callBackRequestDTO.getCode());
        mgRnrTaskDTO.setBizCode("asyncOrder");
        mgRnrTaskDTO.setBizContent(JSONObject.toJSONString(callBackRequestDTO));
        mgRnrTaskDTO.setBizStatus(0);
        mgRnrTaskService.insert(mgRnrTaskDTO);
        if (null != workBackHandler) {
            try {
                workBackHandler.reviewCallBack(callBackRequestDTO);
                mgRnrTaskDTO.setBizStatus(1);
                mgRnrTaskService.update(mgRnrTaskDTO);
            } catch (Exception e) {
                mgRnrTaskDTO.setBizStatus(2);
                mgRnrTaskService.update(mgRnrTaskDTO);
                log.warn("callBackRequestDTO{},e{}",JSON.toJSONString(callBackRequestDTO),e);
            }
        }
    }
}
