package com.cusc.nirvana.user.rnr.mg.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cusc.nirvana.common.result.PageResult;
import com.cusc.nirvana.rds.mybatis.PageHelper;
import com.cusc.nirvana.user.rnr.mg.constants.CommonDeleteEnum;
import com.cusc.nirvana.user.rnr.mg.converter.MgRnrLiaisonInfoConverter;
import com.cusc.nirvana.user.rnr.mg.dao.MgRnrLiaisonInfoDao;
import com.cusc.nirvana.user.rnr.mg.dao.entity.MgRnrLiaisonInfoPO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrLiaisonInfoDTO;
import com.cusc.nirvana.user.rnr.mg.service.IMgRnrLiaisonInfoService;
import com.cusc.nirvana.user.util.CuscStringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * 实名联系人信息表(MgRnrLiaisonInfo)表服务实现类
 *
 * @author yuy336
 * @since 2022-03-03 19:59:05
 */
@Service
public class MgRnrLiaisonInfoServiceImpl extends ServiceImpl<MgRnrLiaisonInfoDao, MgRnrLiaisonInfoPO>
        implements IMgRnrLiaisonInfoService {

    /**
     * 通过UUID查询单条数据
     *
     * @param bean
     * @return 实例对象
     */
    @Override
    public MgRnrLiaisonInfoDTO getByUuid(MgRnrLiaisonInfoDTO bean) {
        MgRnrLiaisonInfoPO record = this.getPoByUuid(bean.getUuid());
        ;
        return MgRnrLiaisonInfoConverter.INSTANCE.poToDto(record);
    }

    @Override
    public MgRnrLiaisonInfoDTO getByRnrid(MgRnrLiaisonInfoDTO bean) {
        MgRnrLiaisonInfoPO record = this.getPoByRnrid(bean.getRnrId());
        return MgRnrLiaisonInfoConverter.INSTANCE.poToDto(record);
    }

    /**
     * 通过查询条件查询集合数据
     *
     * @param bean
     * @return 集合对象
     */
    @Override
    public List<MgRnrLiaisonInfoDTO> queryByList(MgRnrLiaisonInfoDTO bean) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());

        queryWrapper.eq(CuscStringUtils.isNotEmpty(bean.getRnrId()), "rnr_id", bean.getRnrId());
        queryWrapper.eq(bean.getRnrBizzType() != null, "rnr_bizz_type", bean.getRnrBizzType());
        queryWrapper.eq("tenant_no", bean.getTenantNo());

        queryWrapper.orderByDesc("create_time");
        List<MgRnrLiaisonInfoPO> record = this.list(queryWrapper);
        return MgRnrLiaisonInfoConverter.INSTANCE.poListToDtoList(record);
    }

    /**
     * 分页查询
     *
     * @param bean 筛选条件
     * @return 查询结果
     */
    @Override
    public PageResult<MgRnrLiaisonInfoDTO> queryByPage(MgRnrLiaisonInfoDTO bean) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.orderByDesc("create_time");
        queryWrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());

        Page<MgRnrLiaisonInfoPO> page =
                this.page(new Page<>(bean.getCurrPage(), bean.getPageSize()), queryWrapper);
        return PageHelper.convert(page, MgRnrLiaisonInfoDTO.class);
    }

    /**
     * 新增数据
     *
     * @param bean 实例对象
     * @return 实例对象
     */
    @Override
    @Transactional
    public MgRnrLiaisonInfoDTO insert(MgRnrLiaisonInfoDTO bean) {
        MgRnrLiaisonInfoPO mgRnrLiaisonInfoPO = MgRnrLiaisonInfoConverter.INSTANCE.dtoToPo(bean);
        mgRnrLiaisonInfoPO.setUuid(CuscStringUtils.generateUuid());
        this.save(mgRnrLiaisonInfoPO);
        bean.setUuid(mgRnrLiaisonInfoPO.getUuid());
        return bean;
    }

    /**
     * 修改数据
     *
     * @param bean 实例对象
     * @return 实例对象
     */
    @Override
    @Transactional
    public MgRnrLiaisonInfoDTO update(MgRnrLiaisonInfoDTO bean) {
        MgRnrLiaisonInfoPO mgRnrLiaisonInfoPO = this.getPoByUuid(bean.getUuid());
        if (mgRnrLiaisonInfoPO == null) {
            return null;
        }
        MgRnrLiaisonInfoPO tmpBean = MgRnrLiaisonInfoConverter.INSTANCE.dtoToPo(bean);
        tmpBean.setId(mgRnrLiaisonInfoPO.getId());
        this.updateById(tmpBean);
        return bean;
    }

    /**
     * 通过主键删除数据
     *
     * @param bean 实例对象
     * @return 是否成功
     */
    @Override
    @Transactional
    public boolean deleteById(MgRnrLiaisonInfoDTO bean) {
        MgRnrLiaisonInfoPO mgRnrLiaisonInfoPO = this.getPoByUuid(bean.getUuid());
        if (mgRnrLiaisonInfoPO == null) {
            return false;
        }
        MgRnrLiaisonInfoPO tmpBean = new MgRnrLiaisonInfoPO();
        tmpBean.setId(mgRnrLiaisonInfoPO.getId());
        tmpBean.setIsDelete(CommonDeleteEnum.DELETED.getCode());
        return this.updateById(tmpBean);
    }

    @Override
    public Integer addRnrLiaisonBatch(List<MgRnrLiaisonInfoPO> beanList) {
        return baseMapper.addRnrLiaisonBatch(beanList);
    }

    @Override
    public Integer addRnrLiaisonFromRnrInfo(MgRnrInfoDTO info, List<MgRnrLiaisonInfoDTO> beanList) {
        MgRnrLiaisonInfoPO liaisonPO;
        //List<MgRnrLiaisonInfoPO> liaisonPOList = new ArrayList<>();

        for (MgRnrLiaisonInfoDTO liaisonDTO : beanList) {
//            liaisonDTO.setUuid(CuscStringUtils.generateUuid());
            liaisonDTO.setRnrId(info.getUuid());
            liaisonDTO.setTenantNo(info.getTenantNo());
            liaisonDTO.setRoutingKey(info.getRoutingKey());
            liaisonPO = MgRnrLiaisonInfoConverter.INSTANCE.dtoToPo(liaisonDTO);
            //liaisonPOList.add(liaisonPO);
            this.baseMapper.insert(liaisonPO);
        }


        return beanList.size();
    }

    //-------------------私有方法区-------------------

    /**
     * 通过UUID查询单条数据
     *
     * @param uuid
     * @return 实例对象
     */
    private MgRnrLiaisonInfoPO getPoByUuid(String uuid) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("uuid", uuid);
        queryWrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());
        return this.getOne(queryWrapper);
    }

    /**
     * 根据rnrid查询单条数据
     *
     * @param rnrid
     * @return
     */
    public MgRnrLiaisonInfoPO getPoByRnrid(String rnrid) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("rnr_id", rnrid);
        queryWrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());
        return this.getOne(queryWrapper);
    }
}
