package com.cusc.nirvana.user.rnr.mg.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cusc.nirvana.common.result.PageResult;
import com.cusc.nirvana.rds.mybatis.PageHelper;
import com.cusc.nirvana.user.rnr.mg.constants.CommonDeleteEnum;
import com.cusc.nirvana.user.rnr.mg.converter.MgRnrOperationLogConverter;
import com.cusc.nirvana.user.rnr.mg.dao.MgRnrOperationLogDao;
import com.cusc.nirvana.user.rnr.mg.dao.entity.MgRnrOperationLogPO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrOperationLogDTO;
import com.cusc.nirvana.user.rnr.mg.service.IMgRnrOperationLogService;
import com.cusc.nirvana.user.util.CuscStringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * 实名操作日志，包含rnr_info_oplog(MgRnrOperationLog)表服务实现类
 *
 * @author yuy336
 * @since 2022-03-04 15:07:43
 */
@Service
@Slf4j
public class MgRnrOperationLogServiceImpl extends ServiceImpl<MgRnrOperationLogDao, MgRnrOperationLogPO>
        implements IMgRnrOperationLogService {

    /**
     * 通过UUID查询单条数据
     *
     * @param bean
     * @return 实例对象
     */
    @Override
    public MgRnrOperationLogDTO getByUuid(MgRnrOperationLogDTO bean) {
        MgRnrOperationLogPO record = this.getPoByUuid(bean.getUuid());
        ;
        return MgRnrOperationLogConverter.INSTANCE.poToDto(record);
    }

    /**
     * 通过查询条件查询集合数据
     *
     * @param bean
     * @return 集合对象
     */
    @Override
    public List<MgRnrOperationLogDTO> queryByList(MgRnrOperationLogDTO bean) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());
        queryWrapper.orderByDesc("create_time");
        List<MgRnrOperationLogPO> record = this.list(queryWrapper);
        return MgRnrOperationLogConverter.INSTANCE.poListToDtoList(record);
    }

    /**
     * 分页查询
     *
     * @param bean 筛选条件
     * @return 查询结果
     */
    @Override
    public PageResult<MgRnrOperationLogDTO> queryByPage(MgRnrOperationLogDTO bean) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.orderByDesc("create_time");
        queryWrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());

        Page<MgRnrOperationLogPO> page =
                this.page(new Page<>(bean.getCurrPage(), bean.getPageSize()), queryWrapper);
        return PageHelper.convert(page, MgRnrOperationLogDTO.class);
    }

    /**
     * 新增数据
     *
     * @param bean 实例对象
     * @return 实例对象
     */
    @Override
    @Transactional
    public MgRnrOperationLogDTO insert(MgRnrOperationLogDTO bean) {
        MgRnrOperationLogPO mgRnrOperationLogPO = MgRnrOperationLogConverter.INSTANCE.dtoToPo(bean);
        mgRnrOperationLogPO.setUuid(CuscStringUtils.generateUuid());
        this.save(mgRnrOperationLogPO);
        bean.setUuid(mgRnrOperationLogPO.getUuid());
        return bean;
    }

    /**
     * 修改数据
     *
     * @param bean 实例对象
     * @return 实例对象
     */
    @Override
    @Transactional
    public MgRnrOperationLogDTO update(MgRnrOperationLogDTO bean) {
        MgRnrOperationLogPO mgRnrOperationLogPO = this.getPoByUuid(bean.getUuid());
        if (mgRnrOperationLogPO == null) {
            return null;
        }
        MgRnrOperationLogPO tmpBean = MgRnrOperationLogConverter.INSTANCE.dtoToPo(bean);
        tmpBean.setId(mgRnrOperationLogPO.getId());
        this.updateById(tmpBean);
        return bean;
    }

    @Override
    @Async("logToDBExecutor")
    public void addRnrOperationLogBatch(List<MgRnrOperationLogDTO> beanList) {
        log.info("addRnrOperationLogBatch paramter: {}", beanList);
        baseMapper.addRnrOperationLogBatch(MgRnrOperationLogConverter.INSTANCE.dtoListToPoList(beanList));
    }

    @Override
    @Async("logToDBExecutor")
    public void addRnrOperationLogPOBatch(List<MgRnrOperationLogPO> beanList) {
        log.info("addRnrOperationLogPOBatch paramter: {}", beanList);
        baseMapper.addRnrOperationLogBatch(beanList);
    }

    /**
     * 通过UUID查询单条数据
     *
     * @param uuid
     * @return 实例对象
     */
    private MgRnrOperationLogPO getPoByUuid(String uuid) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("uuid", uuid);
        queryWrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());
        return this.getOne(queryWrapper);
    }


}
