package com.cusc.nirvana.user.rnr.mg.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cusc.nirvana.common.result.PageResult;
import com.cusc.nirvana.rds.mybatis.PageHelper;
import com.cusc.nirvana.user.rnr.mg.constants.CommonDeleteEnum;
import com.cusc.nirvana.user.rnr.mg.converter.MgRnrTaskConverter;
import com.cusc.nirvana.user.rnr.mg.dao.entity.MgRnrTaskPO;
import com.cusc.nirvana.user.rnr.mg.dao.mapper.MgRnrTaskDao;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrTaskDTO;
import com.cusc.nirvana.user.rnr.mg.service.IMgRnrTaskService;
import com.cusc.nirvana.user.util.CuscStringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * (MgRnrTask)表服务实现类
 *
 * @author makejava
 * @since 2022-09-24 11:26:04
 */
@Service
public class MgRnrTaskServiceImpl extends ServiceImpl<MgRnrTaskDao, MgRnrTaskPO> implements IMgRnrTaskService {

    /**
     * 通过UUID查询单条数据
     *
     * @param bean
     * @return 实例对象
     */
    @Override
    public MgRnrTaskDTO getByUuid(MgRnrTaskDTO bean) {
        MgRnrTaskPO record = this.getPoByUuid(bean.getUuid());
        return MgRnrTaskConverter.INSTANCE.poToDto(record);
    }
    
     /**
     * 通过查询条件查询集合数据
     *
     * @param bean
     * @return 集合对象
     */
    @Override
    public List<MgRnrTaskDTO> queryByList(MgRnrTaskDTO bean) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());
        queryWrapper.orderByDesc("create_time");
        List<MgRnrTaskPO> record = this.list(queryWrapper);
        return MgRnrTaskConverter.INSTANCE.poListToDtoList(record);
    }

    /**
     * 分页查询
     *
     * @param bean 筛选条件
     * @return 查询结果
     */
    @Override
    public PageResult<MgRnrTaskDTO> queryByPage(MgRnrTaskDTO bean) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.orderByDesc("create_time");
        queryWrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());
        
       Page<MgRnrTaskPO> page =
                this.page(new Page<>(bean.getCurrPage(), bean.getPageSize()), queryWrapper);
        return PageHelper.convert(page, MgRnrTaskDTO.class);
    }

    /**
     * 新增数据
     *
     * @param bean 实例对象
     * @return 实例对象
     */
    @Override
    @Transactional
    public MgRnrTaskDTO insert(MgRnrTaskDTO bean) {
       MgRnrTaskPO mgRnrTaskPO = MgRnrTaskConverter.INSTANCE.dtoToPo(bean);
	   mgRnrTaskPO.setUuid(CuscStringUtils.generateUuid());
        this.save(mgRnrTaskPO);
		bean.setUuid(mgRnrTaskPO.getUuid());
        return bean;
    }

    /**
     * 修改数据
     *
     * @param bean 实例对象
     * @return 实例对象
     */
    @Override
    @Transactional
    public MgRnrTaskDTO update(MgRnrTaskDTO bean) {
        MgRnrTaskPO mgRnrTaskPO = this.getPoByUuid(bean.getUuid());
		 if(mgRnrTaskPO == null){
            return null;
        }
		MgRnrTaskPO tmpBean = MgRnrTaskConverter.INSTANCE.dtoToPo(bean);
		tmpBean.setId(mgRnrTaskPO.getId());
        this.updateById(tmpBean);
        return bean;
    }

    /**
     * 通过主键删除数据
     * @param bean 实例对象
     * @return 是否成功
     */
    @Override
    @Transactional
    public boolean deleteById(MgRnrTaskDTO bean) {
        MgRnrTaskPO mgRnrTaskPO = this.getPoByUuid(bean.getUuid());
		 if (mgRnrTaskPO == null) {
            return false;
        }
        MgRnrTaskPO tmpBean = new MgRnrTaskPO();
        tmpBean.setId(mgRnrTaskPO.getId());
        tmpBean.setIsDelete(CommonDeleteEnum.DELETED.getCode());
        return this.updateById(tmpBean);
    }
    
    /**
     * 通过UUID查询单条数据
     *
     * @param uuid
     * @return 实例对象
     */
    private MgRnrTaskPO getPoByUuid(String uuid) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("uuid", uuid);
        queryWrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());
        return this.getOne(queryWrapper);
    }
}
