package com.cusc.nirvana.user.rnr.mg.service.impl;

import com.alibaba.fastjson.JSON;
import com.cache.redis.util.FastJsonUtils;
import com.cusc.nirvana.common.loader.CollectionUtils;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.eiam.dto.OrganizationDTO;
import com.cusc.nirvana.user.exception.CuscUserException;
import com.cusc.nirvana.user.rnr.common.constants.ResponseCode;
import com.cusc.nirvana.user.rnr.mg.constants.MessageCallbackStatusEnum;
import com.cusc.nirvana.user.rnr.mg.constants.NoticeStatusEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrBizzTypeEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrOptLogBizTypeEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrOptLogOptTypeEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrOptLogSubTypeEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrOrderStatusEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrStatus;
import com.cusc.nirvana.user.rnr.mg.converter.MgRnrCardInfoConverter;
import com.cusc.nirvana.user.rnr.mg.dao.entity.MgRnrCardInfoPO;
import com.cusc.nirvana.user.rnr.mg.dao.entity.MgRnrOperationLogPO;
import com.cusc.nirvana.user.rnr.mg.dto.InfoLogCommentDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgCardNoticeDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrCardInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.RnrOrderDTO;
import com.cusc.nirvana.user.rnr.mg.dto.RnrRelationDTO;
import com.cusc.nirvana.user.rnr.mg.dto.SurplusConfirmDto;
import com.cusc.nirvana.user.rnr.mg.service.*;
import com.cusc.nirvana.user.util.CuscStringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.UUID;

/**
 * @author yubo
 * @since 2022-05-03 15:43
 */
@Service
@Slf4j
public class MgSurplusBindServiceImpl implements IMgSurplusBindService {

    @Autowired
    private IRnrOrderService rnrOrderService;

    @Autowired
    private IMgRnrCardInfoService rnrCardInfoService;

    @Autowired
    private IMgRnrOperationLogService rnrOperationLogService;

    @Autowired
    private IMgRnrCardNoticeService cardNoticeService;

    @Autowired
    private IOrganService organService;

    @Autowired
    private IMqSendService mqSendService;

    @Autowired
    private NotifyCloudService notifyCloudService;

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public String saveRelation(RnrRelationDTO relationDTO) {
        log.info("Mg realtionInfo:" + JSON.toJSONString(relationDTO));

        if (CollectionUtils.isEmpty(relationDTO.getCardList())) {
            throw new CuscUserException(ResponseCode.ICCID_NOT_NULL.getCode(), ResponseCode.ICCID_NOT_NULL.getMsg());
        }

        MgRnrInfoDTO info = relationDTO.getInfo();
        RnrOrderDTO order = relationDTO.getOrder();

        //设置订单信息
        order.setRnrId(info.getUuid());

        //日志信息
        InfoLogCommentDTO comment = new InfoLogCommentDTO();
        comment.setUserId(info.getUuid());
        comment.setLang(LocaleContextHolder.getLocale().toString());
        String logComment = FastJsonUtils.convertObjectToJSON(comment);
        //日志信息
        List<MgRnrOperationLogPO> cardLogList = new ArrayList<>();
        MgRnrOperationLogPO rnrOptLog;

        //组装卡信息
        List<MgRnrCardInfoPO> cardPOList = new ArrayList<>();
        MgRnrCardInfoPO cardInfoPO;

        for (MgRnrCardInfoDTO cardDTO : relationDTO.getCardList()) {
            //设置卡信息
            cardDTO.setUuid(CuscStringUtils.generateUuid());
            cardDTO.setRnrId(info.getUuid());
            cardDTO.setTenantNo(info.getTenantNo());
            cardDTO.setRoutingKey(info.getRoutingKey());
            cardDTO.setOrderId(order.getUuid());

            cardInfoPO = MgRnrCardInfoConverter.INSTANCE.dtoToPo(cardDTO);
            cardPOList.add(cardInfoPO);

            //设置日志信息
            rnrOptLog = new MgRnrOperationLogPO();
            rnrOptLog.setUuid(CuscStringUtils.generateUuid());
            rnrOptLog.setBizType(RnrOptLogBizTypeEnum.CARD.getCode());
            rnrOptLog.setBizUuid(cardDTO.getUuid());
            rnrOptLog.setOptType(RnrOptLogOptTypeEnum.OPERATE_RNR.getCode());
            rnrOptLog.setOptSubType(RnrOptLogSubTypeEnum.CREATE_OPERATE.getCode());
            rnrOptLog.setOptTime(new Date());
            rnrOptLog.setContent(logComment);
            rnrOptLog.setIsBatchOrder(order.getIsBatchOrder());
            rnrOptLog.setSource(order.getOrderSource());
            rnrOptLog.setSerialNumber(order.getSerialNumber());
            rnrOptLog.setCreator(info.getCreator());
            rnrOptLog.setTenantNo(order.getTenantNo());
            cardLogList.add(rnrOptLog);
        }

        //保存卡信息
        rnrCardInfoService.addRnrCardBatch(cardPOList);

        //保存工单信息
        rnrOrderService.insertOrder(order);

        //发送mq
        mqSendService.sendBindUnbindExchangeMq(order.getUuid());

        //保存操作日志-异步
        rnrOperationLogService.addRnrOperationLogPOBatch(cardLogList);

        return order.getUuid();
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void surplusConfirm(SurplusConfirmDto confirmDto) {

        RnrOrderDTO rnrOrderDTO = new RnrOrderDTO();
        rnrOrderDTO.setUuid(confirmDto.getOrderId());
        rnrOrderDTO = rnrOrderService.getByUuid(rnrOrderDTO);
        if (rnrOrderDTO == null) {
            throw new CuscUserException("", "未查询到工单信息:" + confirmDto.getOrderId());
        }

        //如果工单已经完成直接返回
        if (!RnrOrderStatusEnum.notFinished(rnrOrderDTO.getOrderStatus())) {
            return;
        }

        int rnrStatus = RnrStatus.RNR_FAIL.getCode();
        int orderStatus = RnrOrderStatusEnum.CANCEL.getCode();
        int noticeStatus = NoticeStatusEnum.NONEED.getCode();
        if (confirmDto.getCallbackStatusEnum() == MessageCallbackStatusEnum.Y.getCode()) {
            rnrStatus = RnrStatus.RNR.getCode();
            orderStatus = RnrOrderStatusEnum.PASS.getCode();
            noticeStatus = NoticeStatusEnum.NEED.getCode();
        } else if (confirmDto.getCallbackStatusEnum() == MessageCallbackStatusEnum.N.getCode()) {
            rnrStatus = RnrStatus.RNR_FAIL.getCode();
            orderStatus = RnrOrderStatusEnum.NOT_PASS.getCode();
            noticeStatus = NoticeStatusEnum.NONEED.getCode();
        }

        //成功，需要发送mq消息
        List<MgCardNoticeDTO> mqList = new ArrayList<>();
        if (confirmDto.getCallbackStatusEnum() == MessageCallbackStatusEnum.Y.getCode()) {

            //查询车企信息
            String manufacturerOrgId;
            OrganizationDTO organ = new OrganizationDTO();
            organ.setTenantNo(rnrOrderDTO.getTenantNo());
            organ.setUuid(rnrOrderDTO.getOrgId());
            Response<OrganizationDTO> organResp = organService.getCarSubOrganByOrgId(organ);
            if (organResp != null && organResp.isSuccess() && organResp.getData() != null) {
                manufacturerOrgId = organResp.getData().getUuid();
            }else{
                log.warn("surplusConfirm 获取车企组织失败，{}", JSON.toJSONString(organResp));
                manufacturerOrgId = "0";
            }


            MgCardNoticeDTO mqDTO;
            MgRnrCardInfoDTO cardInfoDTO = new MgRnrCardInfoDTO();
            cardInfoDTO.setOrderId(confirmDto.getOrderId());
            List<MgRnrCardInfoDTO> mgRnrCardInfoDTOS = rnrCardInfoService.queryByList(cardInfoDTO);

            for (MgRnrCardInfoDTO cardDTO : mgRnrCardInfoDTOS) {
                //设置消息信息
                mqDTO = new MgCardNoticeDTO();
                mqDTO.setVin(cardDTO.getIotId());
                mqDTO.setRnrBizzType(RnrBizzTypeEnum.Bind.getCode());
                mqDTO.setOrderId(cardDTO.getOrderId());
                mqDTO.setIccid(cardDTO.getIccid());
                mqDTO.setManufacturerOrgId(manufacturerOrgId);
                mqDTO.setResultCode(orderStatus);
                mqDTO.setTenantNo(rnrOrderDTO.getTenantNo());
                mqDTO.setOrgId(rnrOrderDTO.getOrgId());
                mqList.add(mqDTO);
            }
            //通知云平台
            notifyCloudService.notifyResult(mgRnrCardInfoDTOS, RnrBizzTypeEnum.Bind.getCode().toString(), UUID.randomUUID().toString()
                    , null, RnrOrderStatusEnum.PASS.getCode().toString());
        }

        //更新卡状态
        rnrCardInfoService.updateCardStatusByOrderId(confirmDto.getOrderId(), noticeStatus, rnrStatus);

        //更新订单状态
        rnrOrderService.updateOrderStatus(Collections.singletonList(confirmDto.getOrderId()), orderStatus);

        //异步发送消息
        if (!CollectionUtils.isEmpty(mqList)) {
            cardNoticeService.sendCardNotice(mqList);
        }
    }
}
