package com.cusc.nirvana.user.rnr.mg.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cusc.nirvana.common.result.PageResult;
import com.cusc.nirvana.rds.mybatis.PageHelper;
import com.cusc.nirvana.user.rnr.mg.constants.CommonDeleteEnum;
import com.cusc.nirvana.user.rnr.mg.constants.CryptKeyHelper;
import com.cusc.nirvana.user.rnr.mg.converter.MgVehicleCompanyConverter;
import com.cusc.nirvana.user.rnr.mg.dao.MgVehicleCompanyDao;
import com.cusc.nirvana.user.rnr.mg.dao.entity.MgVehicleCompanyPO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrCompanyInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgVehicleCompanyDTO;
import com.cusc.nirvana.user.rnr.mg.service.IMgRnrCompanyInfoService;
import com.cusc.nirvana.user.rnr.mg.service.IMgRnrFileService;
import com.cusc.nirvana.user.rnr.mg.service.IMgRnrInfoService;
import com.cusc.nirvana.user.rnr.mg.service.IMgVehicleCompanyService;
import com.cusc.nirvana.user.util.CuscStringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * 车企信息表(FpVehicleCompany)表服务实现类
 *
 * @author yuy336
 * @since 2022-04-19 20:11:33
 */
@Service
public class MgVehicleCompanyServiceImpl extends ServiceImpl<MgVehicleCompanyDao, MgVehicleCompanyPO>
        implements IMgVehicleCompanyService {

    @Autowired
    IMgRnrCompanyInfoService companyInfoService;

    @Autowired
    IMgRnrInfoService rnrInfoService;

    @Autowired
    IMgRnrFileService fileService;

    /**
     * 通过UUID查询单条数据
     *
     * @param bean
     * @return 实例对象
     */
    @Override
    public MgVehicleCompanyDTO getByUuid(MgVehicleCompanyDTO bean) {
        MgVehicleCompanyPO record = this.getPoByUuid(bean.getUuid());
        ;
        return MgVehicleCompanyConverter.INSTANCE.poToDto(record);
    }

    /**
     * 通过UUID查询单条数据
     *
     * @param bean
     * @return 实例对象
     */
    @Override
    public MgVehicleCompanyDTO getVehicleCompany(MgVehicleCompanyDTO bean) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("tenant_no", bean.getTenantNo());
        queryWrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());

        queryWrapper.eq(CuscStringUtils.isNotEmpty(bean.getOrgId()), "org_id",
                CuscStringUtils.isNotEmpty(bean.getOrgId()));

        MgVehicleCompanyPO vehicleCompany = this.getOne(queryWrapper);
        return MgVehicleCompanyConverter.INSTANCE.poToDto(vehicleCompany);
    }

    /**
     * 通过查询条件查询集合数据
     *
     * @param bean
     * @return 集合对象
     */
    @Override
    public List<MgVehicleCompanyDTO> queryByList(MgVehicleCompanyDTO bean) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());
        queryWrapper.orderByDesc("create_time");

        queryWrapper.eq(CuscStringUtils.isNotEmpty(bean.getTenantNo()), "tenant_no", bean.getTenantNo());
        queryWrapper.eq(CuscStringUtils.isNotEmpty(bean.getCompanyName()), "company_name", CryptKeyHelper.encrypt(bean.getCompanyName()));
        List<MgVehicleCompanyPO> record = this.list(queryWrapper);
        return MgVehicleCompanyConverter.INSTANCE.poListToDtoList(record);
    }

    /**
     * 分页查询
     *
     * @param bean 筛选条件
     * @return 查询结果
     */
    @Override
    public PageResult<MgVehicleCompanyDTO> queryByPage(MgVehicleCompanyDTO bean) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.orderByDesc("create_time");
        queryWrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());

        Page<MgVehicleCompanyPO> page =
                this.page(new Page<>(bean.getCurrPage(), bean.getPageSize()), queryWrapper);
        return PageHelper.convert(page, MgVehicleCompanyDTO.class);
    }

    /**
     * 新增数据
     *
     * @param bean 实例对象
     * @return 实例对象
     */
    @Override
    @Transactional
    public MgVehicleCompanyDTO insert(MgVehicleCompanyDTO bean) {
        MgVehicleCompanyPO vehicleCompanyPO = MgVehicleCompanyConverter.INSTANCE.dtoToPo(bean);
        vehicleCompanyPO.setUuid(CuscStringUtils.generateUuid());
        this.save(vehicleCompanyPO);
        bean.setUuid(vehicleCompanyPO.getUuid());
        return bean;
    }

    /**
     * 修改数据
     *
     * @param bean 实例对象
     * @return 实例对象
     */
    @Override
    @Transactional
    public MgVehicleCompanyDTO update(MgVehicleCompanyDTO bean) {
        MgVehicleCompanyPO MgVehicleCompanyPO = this.getPoByUuid(bean.getUuid());
        if (MgVehicleCompanyPO == null) {
            return null;
        }
        MgVehicleCompanyPO tmpBean = MgVehicleCompanyConverter.INSTANCE.dtoToPo(bean);
        tmpBean.setId(MgVehicleCompanyPO.getId());
        this.updateById(tmpBean);
        return bean;
    }

    /**
     * 通过主键删除数据
     *
     * @param bean 实例对象
     * @return 是否成功
     */
    @Override
    @Transactional
    public boolean deleteById(MgVehicleCompanyDTO bean) {
        MgVehicleCompanyPO MgVehicleCompanyPO = this.getPoByUuid(bean.getUuid());
        if (MgVehicleCompanyPO == null) {
            return false;
        }
        MgVehicleCompanyPO tmpBean = new MgVehicleCompanyPO();
        tmpBean.setId(MgVehicleCompanyPO.getId());
        tmpBean.setIsDelete(CommonDeleteEnum.DELETED.getCode());
        return this.updateById(tmpBean);
    }

    @Override
    @Transactional
    public MgVehicleCompanyDTO saveByRnrId(MgVehicleCompanyDTO bean) {
        //通过rnrId查询企业信息
        MgRnrCompanyInfoDTO company = new MgRnrCompanyInfoDTO();
        company.setRnrId(bean.getFirstRnrId());
        company = companyInfoService.getByRnrid(company);
        if (company == null) {
            return null;
        }
        bean.setCompanyName(company.getCompanyName());
        bean.setCompanyCertType(company.getCompanyCertType());
        bean.setCompanyCertNumber(company.getCompanyCertNumber());
        this.insert(bean);
        return bean;
    }

    /**
     * 通过UUID查询单条数据
     *
     * @param uuid
     * @return 实例对象
     */
    private MgVehicleCompanyPO getPoByUuid(String uuid) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("uuid", uuid);
        queryWrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());
        return this.getOne(queryWrapper);
    }
}
