package com.cusc.nirvana.user.rnr.mg.service.impl;

import com.cache.CacheFactory;
import com.cache.service.ISortSetService;
import com.cusc.nirvana.user.exception.CuscUserException;
import com.cusc.nirvana.user.rnr.mg.constants.RnrMgMqConstant;
import com.cusc.nirvana.user.rnr.mg.service.IMqSendService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;

/**
 * @author yubo
 * @since 2022-05-04 11:03
 */
@Service
@Slf4j
public class MqSendServiceImpl implements IMqSendService {

    @Value("${rnr.mq.bind.delay:180}")
    private int delayTime;

    @Resource
    private CacheFactory cacheFactory;

    /**
     * 发送一车多卡、换卡消息
     *
     * @param orderId
     *//*
    @Override
    public void sendBindUnbindExchangeMq(String orderId) {
        MgBizNoticeDTO mgBizNoticeDTO = new MgBizNoticeDTO(orderId);
        int delayLevel = RnrMqUtil.getLevel(delayTime);

        SendResult sendResult = rocketMQTemplate.syncSend(RnrMgMqConstant.RNR_CARD_OPERATION_DELAY_TOPIC
                , MessageBuilder.withPayload(mgBizNoticeDTO).build(), RnrMqUtil.MQ_TIME_OUT, delayLevel);
        if (sendResult.getSendStatus() != SendStatus.SEND_OK) {
            throw new CuscUserException("", "发送延迟消息失败:" + orderId + ":" + sendResult.toString());
        }
        log.info("发送延迟消息成功:{},{},{}", JSON.toJSONString(mgBizNoticeDTO), delayLevel, sendResult.toString());
    }*/

    /**
     * 发送一车多卡、换卡消息
     *
     * @param orderId
     */
    @Override
    public void sendBindUnbindExchangeMq(String orderId) {
        try {
            //将消息存入redis sortSet,时间戳+过期时间作为score值
            ISortSetService sortSetService = cacheFactory.getSortSetService();
            Map<String, Double> map = new HashMap<>();
            map.put(orderId, ((double) (System.currentTimeMillis() + delayTime * 1000)));
            sortSetService.addSortSet("CT:USER:" + RnrMgMqConstant.RNR_CARD_OPERATION_DELAY_TOPIC, map);
        } catch (Exception e) {
            throw new CuscUserException("", "消息存入redis失败:" + orderId);
        }
    }
}
