package com.cusc.nirvana.user.rnr.mg.service.impl;

import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.rnr.common.cloud.CloudMethod;
import com.cusc.nirvana.user.rnr.common.cloud.CloudService;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrCardInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.UploadResultDto;
import com.cusc.nirvana.user.rnr.mg.dto.VinCardDTO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
@Slf4j
public class NotifyCloudService {

    @Autowired
    private CloudService cloudService;

    @Async
    public Response notifyResult(List<MgRnrCardInfoDTO> dtoList, String rnrBizzType, String serialNumber, String orgId, String resultCode) {
        Response response = null;
        try {
            //将实名结果回传给智网（新增）
            List<VinCardDTO> vinCardList = new ArrayList<>();
            dtoList.forEach(mgRnrCardInfoDTO ->{
                VinCardDTO cardDto = new VinCardDTO();
                cardDto.setIccid(mgRnrCardInfoDTO.getIccid());
                cardDto.setVin(mgRnrCardInfoDTO.getIotId());
                cardDto.setOldIccid(mgRnrCardInfoDTO.getOldCardId());
                cardDto.setResultCode(resultCode);
                vinCardList.add(cardDto);
            });
            UploadResultDto resultDto = new UploadResultDto();
            resultDto.setSerialNumber(serialNumber);
            resultDto.setRnrBizzType(rnrBizzType);
            resultDto.setVinCardList(vinCardList);
            resultDto.setOrgId(orgId);
            log.info("通知云端,实名请求是 = {}", resultDto);
            response =  cloudService.postForResponse(CloudMethod.NOTIFY_RESULT, resultDto, String.class);
            log.info("通知云端,返回结果是 = {}", response);
            if(!response.isSuccess()){
                log.error("通知云端失败,serialNumber = {}", serialNumber);
            }else{
                // todo 保存记录,可以添加重试机制
            }
        }catch (Exception e){
            log.error("实名结果通知云端出现异常 = {}", e.getMessage());
        }

        return response;
    }
}
