package com.cusc.nirvana.user.rnr.mg.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cusc.nirvana.common.result.PageResult;
import com.cusc.nirvana.rds.mybatis.PageHelper;
import com.cusc.nirvana.user.rnr.mg.constants.CommonDeleteEnum;
import com.cusc.nirvana.user.rnr.mg.converter.OpenTenantRelationConverter;
import com.cusc.nirvana.user.rnr.mg.dao.OpenTenantRelationDao;
import com.cusc.nirvana.user.rnr.mg.dao.entity.OpenTenantRelationPO;
import com.cusc.nirvana.user.rnr.mg.dto.OpenTenantRelationDTO;
import com.cusc.nirvana.user.rnr.mg.service.IOpenTenantRelationService;
import com.cusc.nirvana.user.util.CuscStringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * 开放平台租户和实名制租户映射关系表(OpenTenantRelation)表服务实现类
 *
 * @author yuy336
 * @since 2022-06-15 20:03:28
 */
@Service
public class OpenTenantRelationServiceImpl extends ServiceImpl<OpenTenantRelationDao, OpenTenantRelationPO> implements IOpenTenantRelationService {

    /**
     * 通过UUID查询单条数据
     *
     * @param bean
     * @return 实例对象
     */
    @Override
    public OpenTenantRelationDTO getByUuid(OpenTenantRelationDTO bean) {
        OpenTenantRelationPO record = this.getPoByUuid(bean.getUuid());;
        return OpenTenantRelationConverter.INSTANCE.poToDto(record);
    }
    
     /**
     * 通过查询条件查询集合数据
     *
     * @param bean
     * @return 集合对象
     */
    @Override
    public List<OpenTenantRelationDTO> queryByList(OpenTenantRelationDTO bean) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());
        queryWrapper.orderByDesc("create_time");

        queryWrapper.eq(CuscStringUtils.isNotEmpty(bean.getOpenTenantId()), "open_tenant_id", bean.getOpenTenantId());
        queryWrapper.eq("tenant_no", bean.getTenantNo());

        List<OpenTenantRelationPO> record = this.list(queryWrapper);
        return OpenTenantRelationConverter.INSTANCE.poListToDtoList(record);
    }

    /**
     * 分页查询
     *
     * @param bean 筛选条件
     * @return 查询结果
     */
    @Override
    public PageResult<OpenTenantRelationDTO> queryByPage(OpenTenantRelationDTO bean) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.orderByDesc("create_time");
        queryWrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());

        queryWrapper.eq(CuscStringUtils.isNotEmpty(bean.getOpenTenantId()), "open_tenant_id", bean.getOpenTenantId());
        queryWrapper.eq("tenant_no", bean.getTenantNo());
        
       Page<OpenTenantRelationPO> page =
                this.page(new Page<>(bean.getCurrPage(), bean.getPageSize()), queryWrapper);
        return PageHelper.convert(page, OpenTenantRelationDTO.class);
    }

    /**
     * 新增数据
     *
     * @param bean 实例对象
     * @return 实例对象
     */
    @Override
    @Transactional
    public OpenTenantRelationDTO insert(OpenTenantRelationDTO bean) {
       OpenTenantRelationPO openTenantRelationPO = OpenTenantRelationConverter.INSTANCE.dtoToPo(bean);
	   openTenantRelationPO.setUuid(CuscStringUtils.generateUuid());
        this.save(openTenantRelationPO);
		bean.setUuid(openTenantRelationPO.getUuid());
        return bean;
    }

    /**
     * 修改数据
     *
     * @param bean 实例对象
     * @return 实例对象
     */
    @Override
    @Transactional
    public OpenTenantRelationDTO update(OpenTenantRelationDTO bean) {
        OpenTenantRelationPO openTenantRelationPO = this.getPoByUuid(bean.getUuid());
		 if(openTenantRelationPO == null){
            return null;
        }
		OpenTenantRelationPO tmpBean = OpenTenantRelationConverter.INSTANCE.dtoToPo(bean);
		tmpBean.setId(openTenantRelationPO.getId());
        this.updateById(tmpBean);
        return bean;
    }

    /**
     * 通过主键删除数据
     * @param bean 实例对象
     * @return 是否成功
     */
    @Override
    @Transactional
    public boolean deleteById(OpenTenantRelationDTO bean) {
        OpenTenantRelationPO openTenantRelationPO = this.getPoByUuid(bean.getUuid());
		 if (openTenantRelationPO == null) {
            return false;
        }
        OpenTenantRelationPO tmpBean = new OpenTenantRelationPO();
        tmpBean.setId(openTenantRelationPO.getId());
        tmpBean.setIsDelete(CommonDeleteEnum.DELETED.getCode());
        return this.updateById(tmpBean);
    }
    
    /**
     * 通过UUID查询单条数据
     *
     * @param uuid
     * @return 实例对象
     */
    private OpenTenantRelationPO getPoByUuid(String uuid) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("uuid", uuid);
        queryWrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());
        return this.getOne(queryWrapper);
    }
}
