package com.cusc.nirvana.user.rnr.mg.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cusc.nirvana.common.result.PageResult;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.rds.mybatis.PageHelper;
import com.cusc.nirvana.user.rnr.mg.constants.BindingStatus;
import com.cusc.nirvana.user.rnr.mg.constants.CommonDeleteEnum;
import com.cusc.nirvana.user.rnr.mg.converter.OrgSimRelConverter;
import com.cusc.nirvana.user.rnr.mg.dao.OrgSimRelDao;
import com.cusc.nirvana.user.rnr.mg.dao.entity.OrgSimRelPO;
import com.cusc.nirvana.user.rnr.mg.dto.IccIdListRequestDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrCardInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.OrgSimRelDTO;
import com.cusc.nirvana.user.rnr.mg.dto.OrgSimRelQueryDTO;
import com.cusc.nirvana.user.rnr.mg.service.IOrgSimRelService;
import com.cusc.nirvana.user.util.CuscStringUtils;
import com.cusc.nirvana.user.util.DateUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.servlet.support.BindStatus;

import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

@Slf4j
@Service
public class OrgSimRelServiceImpl extends ServiceImpl<OrgSimRelDao, OrgSimRelPO>
        implements IOrgSimRelService {

    @Override
    public OrgSimRelDTO getByUuid(String uuid) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("uuid", uuid);
        queryWrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());
        OrgSimRelPO simRelPO = this.getOne(queryWrapper);
        return simRelPO == null ? null : OrgSimRelConverter.INSTANCE.poToDto(simRelPO);
    }

    @Override
    public OrgSimRelDTO getByIccid(String iccid) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("iccid", iccid);
        queryWrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());
        OrgSimRelPO simRelPO = this.getOne(queryWrapper);
        return simRelPO == null ? null : OrgSimRelConverter.INSTANCE.poToDto(simRelPO);
    }

    @Override
    public PageResult<OrgSimRelDTO> queryByPage(OrgSimRelQueryDTO queryDTO) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("org_uuid", queryDTO.getOrgUuid());
        if (CuscStringUtils.isNotEmpty(queryDTO.getTagUuid())) {
            queryWrapper.eq("tag_code", queryDTO.getTagUuid());
        }
        if (CuscStringUtils.isNotEmpty(queryDTO.getIccid())) {
            queryWrapper.eq("iccid", queryDTO.getIccid());
        }
        if (CuscStringUtils.isNotEmpty(queryDTO.getVin())) {
            queryWrapper.eq("vin", queryDTO.getVin());
        }
        if (null != queryDTO.getBindStatus()) {
            queryWrapper.eq("bind_status", queryDTO.getBindStatus());
        }
        if (queryDTO.getStartCreateTime() != null) {
            queryWrapper.ge("create_time",  DateUtils.formatDate(DateUtils.convertDateToLDT(queryDTO.getStartCreateTime()), "yyyy-MM-dd"));
        }
        if (queryDTO.getEndCreateTime() != null) {
            queryWrapper.le("create_time",
                    DateUtils.formatDate(DateUtils.convertDateToLDT(queryDTO.getEndCreateTime()), "yyyy-MM-dd")+" "
                            + "23:59:59");
        }
        queryWrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());
        queryWrapper.orderByDesc("create_time");
        Page<OrgSimRelDTO> poPage = this.page(new Page<>(queryDTO.getCurrPage(), queryDTO.getPageSize()), queryWrapper);
        return PageHelper.convert(poPage, OrgSimRelDTO.class);
    }

    @Override
    public List<OrgSimRelDTO> queryByList(List<String> iccids) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.in("iccid", iccids);
        queryWrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());
        return this.list(queryWrapper);
    }

    @Override
    public List<OrgSimRelDTO> queryByVin(String vin) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("vin", vin);
        queryWrapper.eq("bind_status", "0");
        queryWrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());
        return this.list(queryWrapper);
    }

    @Override
    public OrgSimRelDTO insert(OrgSimRelDTO simRelDTO) {
        OrgSimRelDTO tmpDto = getByIccid(simRelDTO.getIccid());
        if (tmpDto != null) {
            return tmpDto;
        }
        simRelDTO.setUuid(CuscStringUtils.generateUuid());
        OrgSimRelPO orgSimRelPO = OrgSimRelConverter.INSTANCE.dtoToPo(simRelDTO);
        this.save(orgSimRelPO);
        return simRelDTO;
    }

    @Override
    public Integer insertBatch(List<OrgSimRelDTO> simRelDTOS) {
        simRelDTOS.forEach(dto -> {
            OrgSimRelDTO tmpDto = getByIccid(dto.getIccid());
            if (tmpDto == null) {
                dto.setUuid(CuscStringUtils.generateUuid());
            } else {
                dto.setUuid(tmpDto.getUuid());
                dto.setId(tmpDto.getId());
            }
        });
        boolean flag = true;
        if (!CollectionUtils.isEmpty(simRelDTOS)) {
            flag = this.saveOrUpdateBatch(OrgSimRelConverter.INSTANCE.dtoListToPoList(simRelDTOS));
        }
        return flag ? simRelDTOS.size() : 0;
    }

    @Override
    public OrgSimRelDTO update(OrgSimRelDTO simRelDTO) {
        OrgSimRelDTO bean = this.getByUuid(simRelDTO.getUuid());
        if (bean == null) {
            return null;
        }
        simRelDTO.setId(bean.getId());
        OrgSimRelPO orgSimRelPO = OrgSimRelConverter.INSTANCE.dtoToPo(simRelDTO);
        this.updateById(orgSimRelPO);
        return simRelDTO;
    }

    @Override
    public boolean updateBindStatus(String iccid, String vin, Integer bindStatus) {
		OrgSimRelPO updatePo = new OrgSimRelPO();
		updatePo.setBindStatus(bindStatus);

        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("iccid", iccid);
        wrapper.eq("vin", vin);
        wrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());
        return this.update(updatePo,wrapper);
    }

    @Override
    public boolean updateBindIccid(List<String> iccid) {
        OrgSimRelPO updatePo = new OrgSimRelPO();
//        updatePo.setBindStatus(1);
        //卡状态不改，将VIN置空
        updatePo.setVin("");
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.in("iccid", iccid);
        wrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());
        return this.update(updatePo,wrapper);
    }

    @Override
    public boolean deleteById(OrgSimRelDTO simRelDTO) {
        OrgSimRelDTO dto = this.getByUuid(simRelDTO.getUuid());
        if (dto == null || !dto.getOrgUuid().equals(simRelDTO.getOrgUuid())) {
            return false;
        }
        simRelDTO.setIsDelete(1);
        return this.updateById(OrgSimRelConverter.INSTANCE.dtoToPo(simRelDTO));
    }

    @Async("updateVinExecutor")
    @Override
    public void updateVin(List<MgRnrCardInfoDTO> mgRnrCardInfoDTOList) {
        List<String> iccidList = mgRnrCardInfoDTOList.stream().map(a -> a.getIccid()).collect(Collectors.toList());

        Map<String, MgRnrCardInfoDTO> map = mgRnrCardInfoDTOList.stream().collect(Collectors.toMap(MgRnrCardInfoDTO::getIccid, Function.identity(), (k, v) -> v));

        //根据iccid查询出卡信息
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.in("iccid", iccidList);
        queryWrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());
        List<OrgSimRelPO> simRelPO = this.list(queryWrapper);

        if (CollectionUtils.isEmpty(simRelPO)) {
            log.warn("根据iccid没有查询到卡信息,iccid为{}", JSON.toJSON(iccidList));
        }

        simRelPO.stream().forEach(sim -> {
            MgRnrCardInfoDTO mgRnrCardInfoDTO = map.get(sim.getIccid());
            if (null != mgRnrCardInfoDTO && StringUtils.isEmpty(sim.getVin())) {
                OrgSimRelPO updatePo = new OrgSimRelPO();
                updatePo.setVin(mgRnrCardInfoDTO.getIotId());
                QueryWrapper wrapper = new QueryWrapper();
                wrapper.eq("iccid", sim.getIccid());
                wrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());
                Boolean updateStatus = this.update(updatePo, wrapper);
                if (!updateStatus) {
                    log.warn("根据iccid更新卡信息失败,iccid为{},VIN为{}", sim.getIccid(), sim.getVin());
                }
            } else if (null != mgRnrCardInfoDTO && !StringUtils.isEmpty(sim.getVin()) && !sim.getVin().equals(mgRnrCardInfoDTO.getIotId())) {
                log.info("vin存在且vin不相同,iccid为{},输入VIN为{},数据库中VIN", sim.getIccid(), mgRnrCardInfoDTO.getIotId(), sim.getVin());

            }

        });
    }

    @Override
    public List<OrgSimRelDTO> queryInfoByIccidAndStatus(IccIdListRequestDTO requestDTO) {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.select("DISTINCT iccid");
        wrapper.in("iccid",requestDTO.getIccids());
        wrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());
        wrapper.eq("bind_status",requestDTO.getRnrStatus());
        return this.list(wrapper);
    }

    @Override
    public boolean updateStatusByIccid(IccIdListRequestDTO requestDTO) {
        OrgSimRelPO updatePo = new OrgSimRelPO();

        updatePo.setBindStatus(requestDTO.getRnrStatus());
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.in("iccid", requestDTO.getIccids());
        wrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());
        return this.update(updatePo,wrapper);
    }
}
