package com.cusc.nirvana.user.rnr.mg.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cusc.nirvana.common.result.PageResult;
import com.cusc.nirvana.rds.mybatis.PageHelper;
import com.cusc.nirvana.user.rnr.mg.constants.CommonDeleteEnum;
import com.cusc.nirvana.user.rnr.mg.converter.RnrNoticeTemplateConfigConverter;
import com.cusc.nirvana.user.rnr.mg.dao.RnrNoticeTemplateConfigDao;
import com.cusc.nirvana.user.rnr.mg.dao.entity.RnrNoticeTemplateConfigPO;
import com.cusc.nirvana.user.rnr.mg.dto.RnrNoticeTemplateConfigDTO;
import com.cusc.nirvana.user.rnr.mg.service.IRnrNoticeTemplateConfigService;
import com.cusc.nirvana.user.util.CuscStringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * 消息模板配置(RnrNoticeTemplateConfig)表服务实现类
 *
 * @author yuy336
 * @since 2022-06-01 11:07:09
 */
@Service
public class RnrNoticeTemplateConfigServiceImpl extends ServiceImpl<RnrNoticeTemplateConfigDao, RnrNoticeTemplateConfigPO> implements IRnrNoticeTemplateConfigService {

    /**
     * 通过UUID查询单条数据
     *
     * @param bean
     * @return 实例对象
     */
    @Override
    public RnrNoticeTemplateConfigDTO getByUuid(RnrNoticeTemplateConfigDTO bean) {
        RnrNoticeTemplateConfigPO record = this.getPoByUuid(bean.getUuid());;
        return RnrNoticeTemplateConfigConverter.INSTANCE.poToDto(record);
    }
    
     /**
     * 通过查询条件查询集合数据
     *
     * @param bean
     * @return 集合对象
     */
    @Override
    public List<RnrNoticeTemplateConfigDTO> queryByList(RnrNoticeTemplateConfigDTO bean) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());
        queryWrapper.orderByDesc("create_time");
        List<RnrNoticeTemplateConfigPO> record = this.list(queryWrapper);
        return RnrNoticeTemplateConfigConverter.INSTANCE.poListToDtoList(record);
    }

    /**
     * 分页查询
     *
     * @param bean 筛选条件
     * @return 查询结果
     */
    @Override
    public PageResult<RnrNoticeTemplateConfigDTO> queryByPage(RnrNoticeTemplateConfigDTO bean) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.orderByDesc("create_time");
        queryWrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());
        
       Page<RnrNoticeTemplateConfigPO> page =
                this.page(new Page<>(bean.getCurrPage(), bean.getPageSize()), queryWrapper);
        return PageHelper.convert(page, RnrNoticeTemplateConfigDTO.class);
    }

    /**
     * 新增数据
     *
     * @param bean 实例对象
     * @return 实例对象
     */
    @Override
    @Transactional
    public RnrNoticeTemplateConfigDTO insert(RnrNoticeTemplateConfigDTO bean) {
       RnrNoticeTemplateConfigPO rnrNoticeTemplateConfigPO = RnrNoticeTemplateConfigConverter.INSTANCE.dtoToPo(bean);
	   rnrNoticeTemplateConfigPO.setUuid(CuscStringUtils.generateUuid());
        this.save(rnrNoticeTemplateConfigPO);
		bean.setUuid(rnrNoticeTemplateConfigPO.getUuid());
        return bean;
    }

    /**
     * 修改数据
     *
     * @param bean 实例对象
     * @return 实例对象
     */
    @Override
    @Transactional
    public RnrNoticeTemplateConfigDTO update(RnrNoticeTemplateConfigDTO bean) {
        RnrNoticeTemplateConfigPO rnrNoticeTemplateConfigPO = this.getPoByUuid(bean.getUuid());
		 if(rnrNoticeTemplateConfigPO == null){
            return null;
        }
		RnrNoticeTemplateConfigPO tmpBean = RnrNoticeTemplateConfigConverter.INSTANCE.dtoToPo(bean);
		tmpBean.setId(rnrNoticeTemplateConfigPO.getId());
        this.updateById(tmpBean);
        return bean;
    }

    /**
     * 通过主键删除数据
     * @param bean 实例对象
     * @return 是否成功
     */
    @Override
    @Transactional
    public boolean deleteById(RnrNoticeTemplateConfigDTO bean) {
        RnrNoticeTemplateConfigPO rnrNoticeTemplateConfigPO = this.getPoByUuid(bean.getUuid());
		 if (rnrNoticeTemplateConfigPO == null) {
            return false;
        }
        RnrNoticeTemplateConfigPO tmpBean = new RnrNoticeTemplateConfigPO();
        tmpBean.setId(rnrNoticeTemplateConfigPO.getId());
        tmpBean.setIsDelete(CommonDeleteEnum.DELETED.getCode());
        return this.updateById(tmpBean);
    }
    
    /**
     * 通过UUID查询单条数据
     *
     * @param uuid
     * @return 实例对象
     */
    private RnrNoticeTemplateConfigPO getPoByUuid(String uuid) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("uuid", uuid);
        queryWrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());
        return this.getOne(queryWrapper);
    }
}
