package com.cusc.nirvana.user.rnr.mg.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cusc.nirvana.common.result.PageResult;
import com.cusc.nirvana.rds.mybatis.PageHelper;
import com.cusc.nirvana.user.exception.CuscUserException;
import com.cusc.nirvana.user.rnr.common.dto.RnrMqMessageDTO;
import com.cusc.nirvana.user.rnr.mg.constants.CommonDeleteEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrOptLogSubTypeEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrOrderStatusEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrOrderType;
import com.cusc.nirvana.user.rnr.mg.converter.RnrOrderConverter;
import com.cusc.nirvana.user.rnr.mg.dao.RnrOrderDao;
import com.cusc.nirvana.user.rnr.mg.dao.entity.RnrOrderOperationLogPO;
import com.cusc.nirvana.user.rnr.mg.dao.entity.RnrOrderPO;
import com.cusc.nirvana.user.rnr.mg.dto.*;
import com.cusc.nirvana.user.rnr.mg.service.IOrderNoticeService;
import com.cusc.nirvana.user.rnr.mg.service.IRnrOrderOperationLogService;
import com.cusc.nirvana.user.rnr.mg.service.IRnrOrderService;
import com.cusc.nirvana.user.rnr.mg.util.crypt.DesensitizationUtil;
import com.cusc.nirvana.user.rnr.workorder.constants.WorkOrderTypeEnum;
import com.cusc.nirvana.user.rnr.workorder.context.WorkOrderContext;
import com.cusc.nirvana.user.rnr.workorder.dto.WorkOrderStarRequestDTO;
import com.cusc.nirvana.user.util.CuscStringUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

/**
 * 实名工单信息(MgRnrOrder)表服务实现类
 *
 * @author yuy336
 * @since 2022-01-26 13:49:52
 */
@Service
@Slf4j
public class RnrOrderServiceImpl extends ServiceImpl<RnrOrderDao, RnrOrderPO> implements IRnrOrderService {

    @Autowired
    private IRnrOrderOperationLogService rnrOrderOperationLogService;

    @Autowired
    private IOrderNoticeService orderNoticeService;

    @Autowired
    private WorkOrderContext workOrderContext;

    @Autowired
    private RnrOrderDao rnrOrderDao;

    @Value("${mg.workOrder.applicationId:112f1c67de134e669379e24f160fb25a}")
    private String applicationId;

    @Value("${mg.workOrder.sendOrderSystemFlag:0}")
    private String sendOrderSystemFlag;

    /**
     * 通过UUID查询单条数据
     *
     * @param bean
     * @return 实例对象
     */
    @Override
    public RnrOrderDTO getByUuid(RnrOrderDTO bean) {
        RnrOrderPO record = this.getPoByUuid(bean.getUuid());
        return RnrOrderConverter.INSTANCE.poToDto(record);
    }

    /**
     * 通过查询条件查询集合数据
     *
     * @param bean
     * @return 集合对象
     */
    @Override
    public List<RnrOrderDTO> queryByList(RnrOrderDTO bean) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());
        queryWrapper.orderByDesc("create_time");

        queryWrapper.eq(CuscStringUtils.isNotEmpty(bean.getRnrId()),"rnr_id",bean.getRnrId());
        queryWrapper.eq(CuscStringUtils.isNotEmpty(bean.getOrderSource()),"order_source",bean.getOrderSource());
        queryWrapper.eq(bean.getRnrBizzType() != null,"rnr_bizz_type",bean.getRnrBizzType());
        queryWrapper.eq(bean.getOrderType() != null,"order_type",bean.getOrderType());
        queryWrapper.eq(bean.getOrderStatus() != null,"order_status",bean.getOrderStatus());
		queryWrapper.in(!CollectionUtils.isEmpty(bean.getUuidList()),"uuid",bean.getUuidList());
		queryWrapper.in(!CollectionUtils.isEmpty(bean.getRnrIdList()),"rnr_id",bean.getRnrIdList());

        List<RnrOrderPO> record = this.list(queryWrapper);
        return RnrOrderConverter.INSTANCE.poListToDtoList(record);
    }

    /**
     * 分页查询
     *
     * @param bean 筛选条件
     * @return 查询结果
     */
    @Override
    public PageResult<RnrOrderDTO> queryByPage(RnrOrderDTO bean) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.orderByDesc("create_time");
        queryWrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());

        queryWrapper.eq(bean.getOrderStatus() != null, "order_status", bean.getOrderStatus());
        queryWrapper.le(bean.getCreateTime() != null, "create_time", bean.getCreateTime());
		queryWrapper.eq(StringUtils.isNotBlank(bean.getTenantNo()),"tenant_no",bean.getTenantNo());
		queryWrapper.eq(null != bean.getOrderType(),"order_type",bean.getOrderType());
		queryWrapper.in(!CollectionUtils.isEmpty(bean.getUuidList()),"uuid",bean.getUuidList());
		queryWrapper.in(!CollectionUtils.isEmpty(bean.getRnrIdList()),"rnr_id",bean.getRnrIdList());
		queryWrapper.gt(null != bean.getCreateTimeQueryStart(),"create_time",bean.getCreateTimeQueryStart());
		queryWrapper.lt(null != bean.getCreateTimeQueryEnd(),"create_time",bean.getCreateTimeQueryEnd());
		queryWrapper.in(!CollectionUtils.isEmpty(bean.getOrgIdList()),"org_id",bean.getOrgIdList());
		queryWrapper.in(!CollectionUtils.isEmpty(bean.getOrderStatusList()),"order_status",bean.getOrderStatusList());
		queryWrapper.in(!CollectionUtils.isEmpty(bean.getOrderTypeList()),"order_type",bean.getOrderTypeList());

        Page<RnrOrderPO> page =
                this.page(new Page<>(bean.getCurrPage(), bean.getPageSize()), queryWrapper);
        return PageHelper.convert(page, RnrOrderDTO.class);
    }

    /**
     * 新增数据
     *
     * @param bean 实例对象
     * @return 实例对象
     */
    @Override
    @Transactional
    public RnrOrderDTO insert(RnrOrderDTO bean) {
        RnrOrderPO rnrOrderPO = RnrOrderConverter.INSTANCE.dtoToPo(bean);
        if(CuscStringUtils.isEmpty(bean.getUuid())){
            rnrOrderPO.setUuid(CuscStringUtils.generateUuid());
            bean.setUuid(rnrOrderPO.getUuid());
        }
        this.save(rnrOrderPO);
        return bean;
    }

    /**
     * 修改数据
     *
     * @param bean 实例对象
     * @return 实例对象
     */
    @Override
    @Transactional
    public RnrOrderDTO update(RnrOrderDTO bean) {
        RnrOrderPO MgRnrOrderPO = this.getPoByUuid(bean.getUuid());
        if (MgRnrOrderPO == null) {
            return null;
        }
        RnrOrderPO tmpBean = RnrOrderConverter.INSTANCE.dtoToPo(bean);
        tmpBean.setId(MgRnrOrderPO.getId());
        this.updateById(tmpBean);
        return bean;
    }

    /**
     * 通过主键删除数据
     *
     * @param bean 实例对象
     * @return 是否成功
     */
    @Override
    @Transactional
    public boolean deleteById(RnrOrderDTO bean) {
        RnrOrderPO MgRnrOrderPO = this.getPoByUuid(bean.getUuid());
        if (MgRnrOrderPO == null) {
            return false;
        }
        RnrOrderPO tmpBean = new RnrOrderPO();
        tmpBean.setId(MgRnrOrderPO.getId());
        tmpBean.setIsDelete(CommonDeleteEnum.DELETED.getCode());
        return this.updateById(tmpBean);
    }

    @Override
    public void sendBatchOrderMQ(List<RnrOrderDTO> orderList, Integer orderDelay) {
        RnrMqMessageDTO<List<RnrOrderDTO>> mqMsg;
        if (orderDelay != null && orderDelay > 0) {
            mqMsg = RnrMqMessageDTO.createDelayMsg(orderDelay, orderList);
        } else {
            mqMsg = RnrMqMessageDTO.createMsg(orderList);
        }
        orderNoticeService.sendOrderMsg(mqMsg);
    }

    @Override
    public void insertOrder(RnrOrderDTO order) {
        // 当前时间
        Date now = new Date();

        RnrOrderPO orderPO = RnrOrderConverter.INSTANCE.dtoToPo(order);

        if (Boolean.TRUE.equals(order.getSendWorkOrder())) {
            orderPO.setOrderStatus(RnrOrderStatusEnum.ASSIGNMENT.getCode());
        } else {
            orderPO.setAuditTime(now);
            orderPO.setAllocateTime(now);
        }

        // 存操作
        RnrOrderOperationLogPO orderOperationLogPO = new RnrOrderOperationLogPO();
        orderOperationLogPO.setUuid(CuscStringUtils.generateUuid());
        orderOperationLogPO.setOrderId(orderPO.getUuid());
        orderOperationLogPO.setCreator(orderPO.getCreator() == null ? "0" : orderPO.getCreator());
        orderOperationLogPO.setOptType(RnrOptLogSubTypeEnum.CREATE_OPERATE.getCode());
        orderOperationLogPO.setUpdatedCode(0L);
        orderOperationLogPO.setUpdatedContent("本单由工号:" + orderPO.getCreator() + "创建");
        orderOperationLogPO.setOptTime(now);
        orderOperationLogPO.setTenantNo(orderPO.getTenantNo());
        orderOperationLogPO.setRnrId(orderPO.getRnrId());

        baseMapper.insert(orderPO);
        rnrOrderOperationLogService.insertBatchOrderOperationLog(Collections.singletonList(orderOperationLogPO));
    }

	@Override
    public Integer insertBatchOrder(List<RnrOrderDTO> orderList) {
        // 当前时间
        Date now = new Date();
        RnrOrderPO orderPO;
        List<RnrOrderPO> orderSaveList = new ArrayList<>();

        RnrOrderOperationLogPO orderOperationLogPO;
        List<RnrOrderOperationLogPO> orderLogList = new ArrayList<>();
//        List<RnrOrderDTO> failedOrderDTOS = new ArrayList<>();

        for (RnrOrderDTO order : orderList) {
            // 存工单
            orderPO = RnrOrderConverter.INSTANCE.dtoToPo(order);
            if (orderPO.getOrderStatus() == RnrOrderStatusEnum.PASS.getCode()) {
                orderPO.setAuditTime(now);
                orderPO.setAllocateTime(now);
            }
            orderSaveList.add(orderPO);

//            //没有审核通过的工单变成待审核状态，进入工单系统，手工审核
//            if (Boolean.TRUE.equals(order.getSendWorkOrder())) {
//                orderPO.setOrderStatus(RnrOrderStatusEnum.ASSIGNMENT.getCode());
//                failedOrderDTOS.add(order);
//            }

            // 存操作
            orderOperationLogPO = new RnrOrderOperationLogPO();
            orderOperationLogPO.setUuid(CuscStringUtils.generateUuid());
            orderOperationLogPO.setCreator(orderPO.getCreator() == null ? "0" : orderPO.getCreator());
            orderOperationLogPO.setOptType(RnrOptLogSubTypeEnum.CREATE_OPERATE.getCode());
            orderOperationLogPO.setUpdatedCode(0L);
            orderOperationLogPO.setUpdatedContent("本单由工号:" + orderPO.getCreator() + "创建");
            orderOperationLogPO.setOptTime(now);
            orderOperationLogPO.setOrderId(orderPO.getUuid());
            orderOperationLogPO.setTenantNo(orderPO.getTenantNo());
            orderOperationLogPO.setRnrId(orderPO.getRnrId());
            orderLogList.add(orderOperationLogPO);
        }
        baseMapper.insertBatchOrder(orderSaveList);
        rnrOrderOperationLogService.insertBatchOrderOperationLog(orderLogList);

//        sendOrderSystem(failedOrderDTOS);
        return orderSaveList.size();
    }

    @Override
    public void sendThirdBatchOrder(List<RnrOrderDTO> orderList) {

    }

    /**
     * 通知工单系统
     */
    @Override
    public void sendOrderSystem(RnrOrderDTO rnrOrderDTO, MgRnrInfoDTO mgRnrInfoDTO, List<MgRnrCardInfoDTO> list) {
        if("0".equals(sendOrderSystemFlag)){
            log.info("work order begin local verify mode, dont send rnrOrderSystem");
            return;
        }
        String name = DesensitizationUtil.desensitizeName(mgRnrInfoDTO.getFullName());
        String phone = DesensitizationUtil.desensitizePhone(mgRnrInfoDTO.getPhone());

        WorkOrderStarRequestDTO orderStarRequestDTO;
        if(CollectionUtils.isEmpty(list)){
            orderStarRequestDTO = orderDtoToWorkOrderDTO(rnrOrderDTO,name,phone,"","");
        }else{
            MgRnrCardInfoDTO mgRnrCardInfoDTO = list.get(0);
            boolean notSingle = list.size() > 1;
            String vin = notSingle ? mgRnrCardInfoDTO.getIotId()+",..." : mgRnrCardInfoDTO.getIotId();
            String iccid = notSingle ? mgRnrCardInfoDTO.getIccid()+",..." : mgRnrCardInfoDTO.getIccid();
            orderStarRequestDTO = orderDtoToWorkOrderDTO(rnrOrderDTO,name,phone,vin,iccid);
        }

//        for (RnrOrderDTO rnrOrderDTO : orderList) {

            log.info("发送工单：{}", JSON.toJSONString(orderStarRequestDTO));
            String workOrderCode = workOrderContext.getHandler().startOrder(orderStarRequestDTO);
            if (!StringUtils.isBlank(workOrderCode)) {
                this.baseMapper.updateOrderApproveNo(rnrOrderDTO.getUuid(), workOrderCode);
            } else {
                log.error("发送工单系统失败,订单UUID:" + rnrOrderDTO.getUuid());
                throw new CuscUserException("", "发送工单系统失败");
            }
//        }
    }

    /**
     * 更新工单状态
     */
    @Override
    @Transactional
    public void updateOrderStatus(List<String> orderIds, int status) {
        this.baseMapper.updateOrderStatus(orderIds, status);
    }

    //转换工单
    private WorkOrderStarRequestDTO orderDtoToWorkOrderDTO(RnrOrderDTO rnrOrderDTO,String... args) {
        WorkOrderStarRequestDTO requestDTO = new WorkOrderStarRequestDTO();
        requestDTO.setBusinessObjectId(rnrOrderDTO.getUuid());
        requestDTO.setTenantUuid(rnrOrderDTO.getTenantNo());
        RnrOrderType typeByCode = RnrOrderType.getTypeByCode(rnrOrderDTO.getOrderType());
        if (typeByCode == null) {
            throw new CuscUserException("", "工单类型错误");
        }
        WorkOrderTypeEnum workOrderTypeEnum
                = WorkOrderTypeEnum.getWorkOrderTypeEnumByRnrOrderType(typeByCode);
        requestDTO.setWorkOrderKey(workOrderTypeEnum.getKey());
        requestDTO.setName(workOrderTypeEnum.getWorkOrderName(args));
        requestDTO.setStartUserId(rnrOrderDTO.getCreator());
        requestDTO.setApplicationUuid(applicationId);
        return requestDTO;
    }

    /**
     * 通过UUID查询单条数据
     *
     * @param uuid
     * @return 实例对象
     */
    public RnrOrderPO getPoByUuid(String uuid) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("uuid", uuid);
        queryWrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());
        return this.getOne(queryWrapper);
    }

    public Integer orderTotal(LocalVerifyListRqDTO rq) {
        return rnrOrderDao.orderTotal(rq);
    }

    public List<LocalVerifyListDTO> orderList(LocalVerifyListRqDTO rq) {
        return rnrOrderDao.orderList(rq, (rq.getCurrPage() - 1) * rq.getPageSize(), rq.getPageSize());
    }

    @Override
    public String getOrganNameForOrgId(String orgId){
        return rnrOrderDao.getOrganNameForOrgId(orgId);
    }

    @Override
    public String getOrganNameForOrganCode(String organCode){
        return rnrOrderDao.getOrganNameForOrganCode(organCode);
    }
}
