package com.cusc.nirvana.user.rnr.mg.service.impl;

import com.alibaba.fastjson.JSON;
import com.cache.CacheFactory;
import com.cache.exception.CacheException;
import com.cache.redis.util.FastJsonUtils;
import com.cusc.nirvana.common.loader.CollectionUtils;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.eiam.dto.OrganizationDTO;
import com.cusc.nirvana.user.exception.CuscUserException;
import com.cusc.nirvana.user.rnr.common.constants.RedisConstant;
import com.cusc.nirvana.user.rnr.common.constants.ResponseCode;
import com.cusc.nirvana.user.rnr.mg.constants.NoticeStatusEnum;
import com.cusc.nirvana.user.rnr.mg.constants.PassCodeEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrBizzTypeEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrOptLogBizTypeEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrOptLogOptTypeEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrOptLogSubTypeEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrOrderAuditTypeEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrOrderSourceEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrOrderStatusEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrOrderType;
import com.cusc.nirvana.user.rnr.mg.constants.RnrResponseCode;
import com.cusc.nirvana.user.rnr.mg.constants.RnrStatus;
import com.cusc.nirvana.user.rnr.mg.converter.MgRnrCardInfoConverter;
import com.cusc.nirvana.user.rnr.mg.dao.OrgSimRelDao;
import com.cusc.nirvana.user.rnr.mg.dao.entity.MgRnrCardInfoPO;
import com.cusc.nirvana.user.rnr.mg.dao.entity.MgRnrOperationLogPO;
import com.cusc.nirvana.user.rnr.mg.dto.InfoLogCommentDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgCardNoticeDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrCardInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrCompanyInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrFileDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrLiaisonInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.RnrOrderDTO;
import com.cusc.nirvana.user.rnr.mg.dto.RnrRelationDTO;
import com.cusc.nirvana.user.rnr.mg.service.*;
import com.cusc.nirvana.user.rnr.mg.util.RnrOrderUtil;
import com.cusc.nirvana.user.rnr.mg.util.SpringValidationUtil;
import com.cusc.nirvana.user.rnr.mg.validation.group.SaveRelation;
import com.cusc.nirvana.user.util.CuscStringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.validation.ConstraintViolation;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Set;

/**
 * Description: 实名关系service
 * <br />
 * CreateDate 2022-01-26 10:36:27
 *
 * @author yuyi
 **/
@Service
@Slf4j
public class RnrRelationServiceImpl implements IRnrRelationService {

    @Autowired
    private IRnrOrderService rnrOrderService;

    @Autowired
    private IMgRnrInfoService rnrInfoService;

    @Autowired
    private IMgRnrCardInfoService rnrCardInfoService;

    @Autowired
    private IMgRnrCompanyInfoService rnrCompanyInfoService;

    @Autowired
    private IMgRnrTagService rnrTagService;

    @Autowired
    private IMgRnrLiaisonInfoService rnrLiaisonInfoService;

    @Autowired
    private IMgRnrFileService rnrFileService;

    @Resource
    private CacheFactory cacheFactory;

    @Autowired
    private IMgRnrOperationLogService rnrOperationLogService;

    @Autowired
    @Lazy
    private IMgRnrCardNoticeService cardNoticeService;

    @Autowired
    private IProjectService projectService;

    @Autowired
    private IMgRnrCompanyInfoService mgRnrCompanyInfoService;

    @Autowired
    private ICiamService ciamService;

    @Autowired
    private IOrganService organService;

    @Resource
    private OrgSimRelDao orgSimRelDao;

	@Resource
	private IRnrUnboundService unboundService;

    @Resource
    private IOrgSimRelService iOrgSimRelService;


    @Override
    @Transactional
    public Response saveRnrRelation(RnrRelationDTO bean) {
        log.info("Mg realtionInfo:" + JSON.toJSONString(bean));

        if (CollectionUtils.isEmpty(bean.getCardList())) {
            return Response.createError(ResponseCode.ICCID_NOT_NULL.getMsg(), ResponseCode.ICCID_NOT_NULL.getCode());
        }

		try {
			unboundService.checkUnBindVehicle(bean);
		} catch (CuscUserException e) {
			throw e;
		} catch (Exception e) {
			log.error("checkUnBindVehicle has error orderId = {}", bean.getOrder().getUuid(), e);
		}

        MgRnrInfoDTO info = bean.getInfo();
        RnrOrderDTO order = bean.getOrder();

//        //通过证件号查询路由键
//        Response<Long> routingKeyResp = systemRoutingService.getOrCreateRoutingKey(info.getCertNumber());
//        if (routingKeyResp == null || !routingKeyResp.isSuccess()) {
//            return routingKeyResp;
//        }

//        info.setRoutingKey(routingKeyResp.getData());

        //设置订单信息
        order.setRnrId(info.getUuid());
//        order.setRoutingKey(routingKeyResp.getData());
        /*order.setFinalApproveTime(new Date(System.currentTimeMillis() + DateUtils.stringHourToMills(RnrConstants.DEFAULT_AUDIT_TIME)));*/
        if (order.isAutoRnr()) {
            order.setUpdatedCode(PassCodeEnum.AUTO_SUCC.getCode());
            if (order.getOrderType().intValue() == RnrOrderType.REBINDING.getCode()) {
                order.setOldOrderStatus(RnrOrderStatusEnum.PASS.getCode());
            }
            order.setAuditType(RnrOrderAuditTypeEnum.AUTO.getCode());
        } /*else {
            order.setAuditType(RnrOrderAuditTypeEnum.MANUAL.getCode());
            order.setOrderStatus(RnrOrderStatusEnum.ASSIGNMENT.getCode());
            info.setRnrStatus(RnrStatus.INIT.getCode());
        }*/
        //todo 改为项目配置
        //String hour = brandRepo.findFinalApproveHourByBrandId(brandId);
        //if (StringUtils.isBlank(hour)) {
        //    hour = CuscRnrConstants.DEFAULT_AUDIT_TIME;
        //}
        //查询车企信息
        OrganizationDTO organ = new OrganizationDTO();
        organ.setTenantNo(info.getTenantNo());
        organ.setUuid(info.getOrgId());
        Response<OrganizationDTO> organResp = organService.getCarSubOrganByOrgId(organ);
        if(organResp == null || !organResp.isSuccess() || organResp.getData() == null){
            return Response.createError(ResponseCode.CAR_SUB_ORGAN_NOT_FOUND.getMsg(), ResponseCode.CAR_SUB_ORGAN_NOT_FOUND.getCode());
        }

        InfoLogCommentDTO comment = new InfoLogCommentDTO();
        comment.setUserId(info.getUuid());
        comment.setLang(LocaleContextHolder.getLocale().toString());
        String logComment = FastJsonUtils.convertObjectToJSON(comment);

        //组装卡信息
        List<MgRnrCardInfoPO> cardPOList = new ArrayList<>();
        MgRnrCardInfoPO cardInfoPO;

        //日志信息
        List<MgRnrOperationLogPO> cardLogList = new ArrayList<>();
        MgRnrOperationLogPO rnrOptLog;

        //mq消息
        List<MgCardNoticeDTO> mqList = new ArrayList<>();
        MgCardNoticeDTO mqDTO;
        List<MgRnrCardInfoDTO> mgRnrCardInfoDTOList = new ArrayList<>();
        MgRnrCardInfoDTO mgRnrCardInfoDTO = new MgRnrCardInfoDTO();
        for (MgRnrCardInfoDTO cardDTO : bean.getCardList()) {
            mgRnrCardInfoDTO.setIccid(cardDTO.getIccid());
            mgRnrCardInfoDTO.setIotId(cardDTO.getIotId());
            mgRnrCardInfoDTOList.add(mgRnrCardInfoDTO);
            //设置卡信息
            cardDTO.setUuid(CuscStringUtils.generateUuid());
            cardDTO.setRnrId(info.getUuid());
            cardDTO.setTenantNo(info.getTenantNo());
            cardDTO.setRoutingKey(info.getRoutingKey());
            cardDTO.setNoticeStatus(NoticeStatusEnum.NONEED.getCode());
            cardDTO.setOrderId(order.getUuid());

            //设置日志信息
            rnrOptLog = new MgRnrOperationLogPO();
            rnrOptLog.setUuid(CuscStringUtils.generateUuid());
            rnrOptLog.setBizType(RnrOptLogBizTypeEnum.CARD.getCode());
            rnrOptLog.setBizUuid(cardDTO.getUuid());
            rnrOptLog.setOptType(RnrOptLogOptTypeEnum.OPERATE_RNR.getCode());
            rnrOptLog.setOptSubType(order.getAuditType().intValue() == RnrOrderAuditTypeEnum.AUTO.getCode() ?
                    RnrOptLogSubTypeEnum.AUTO_ACCESS_OPERATE.getCode() :
                    RnrOptLogSubTypeEnum.RNR_SHIFT_TO_ARTIFICIAL.getCode());
            rnrOptLog.setOptTime(new Date());
            rnrOptLog.setContent(logComment);
            rnrOptLog.setIsBatchOrder(order.getIsBatchOrder());
            rnrOptLog.setSource(order.getOrderSource());
            rnrOptLog.setSerialNumber(order.getSerialNumber());
            rnrOptLog.setCreator(info.getCreator());
            rnrOptLog.setTenantNo(order.getTenantNo());
            cardLogList.add(rnrOptLog);

            //不发送工单系统，则表示自动审核，需要发送mq
            if (!order.getSendWorkOrder() && RnrOrderStatusEnum.PASS.getCode().equals(bean.getOrder().getOrderStatus())) {
                cardDTO.setNoticeStatus(NoticeStatusEnum.NEED.getCode());
                cardDTO.setRnrStatus(RnrOrderUtil.orderTypeToRnrStatus(order.getOrderType()).getCode());

                //设置消息信息
                mqDTO = new MgCardNoticeDTO();
                mqDTO.setVin(cardDTO.getIotId());
                mqDTO.setRnrBizzType(RnrOrderUtil.orderTypeToNoticeType(order.getOrderType()).getCode());
                mqDTO.setOrderId(cardDTO.getOrderId());
                mqDTO.setIccid(cardDTO.getIccid());

                mqDTO.setManufacturerOrgId(organResp.getData().getUuid());
                mqDTO.setResultCode(bean.getOrder().getOrderStatus());
                mqDTO.setTenantNo(info.getTenantNo());
                mqDTO.setOrgId(info.getOrgId());
                mqList.add(mqDTO);
            } else {
                cardDTO.setRnrStatus(RnrStatus.INIT.getCode());
            }

            cardDTO.setRnrId(info.getUuid());
            cardInfoPO = MgRnrCardInfoConverter.INSTANCE.dtoToPo(cardDTO);
            cardPOList.add(cardInfoPO);
        }

        //保存实名表数据
        rnrInfoService.insert(info);

        //保存卡信息
        rnrCardInfoService.addRnrCardBatch(cardPOList);

        //保存tag信息
        if (CollectionUtils.isNotEmpty(bean.getRnrTagList())) {
            rnrTagService.addRnrTagFromRnrInfo(info, bean.getRnrTagList());
        }

        //保存联系人信息
        if (CollectionUtils.isNotEmpty(bean.getRnrLiaisonList())) {
            rnrLiaisonInfoService.addRnrLiaisonFromRnrInfo(info, bean.getRnrLiaisonList());
        }

        //保存公司信息
        if (bean.getCompanyInfo() != null) {
            mgRnrCompanyInfoService.insert(bean.getCompanyInfo());
        }

        //保存图片信息
        rnrFileService.addRnrFileFromInfo(info, bean.getRnrFileList());

        //保存工单信息
        rnrOrderService.insertOrder(order);

        //发送工单系统
        if (order.getSendWorkOrder()) {
            rnrOrderService.sendOrderSystem(order, info, bean.getCardList());
        }

//        mq发送工单
//        rnrOrderService.sendBatchOrderMQ(orderList, Integer.parseInt(order.getOrderDelay()));
        //保存操作日志-异步
        rnrOperationLogService.addRnrOperationLogPOBatch(cardLogList);

        //非委托人模式，自然人新车和二手车成功需要创建用户
        if (!bean.getOrder().getSendWorkOrder() && RnrOrderUtil.needCreateUser(bean.getInfo(), bean.getOrder()) && !RnrOrderSourceEnum.H5_CUSTOMER.getCode().equals(bean.getOrder().getOrderSource())) {
            ciamService.createUser(bean.getInfo());
        }

        //异步发送审核成功消息
        if (CollectionUtils.isNotEmpty(mqList)) {
            //如果是绑定或者解绑需要做一下数据更新处理
            if(RnrBizzTypeEnum.Bind.getCode().equals(order.getRnrBizzType())||RnrBizzTypeEnum.ChangeBinding.getCode().equals(order.getRnrBizzType())){
                iOrgSimRelService.updateVin(mgRnrCardInfoDTOList);
            }
            cardNoticeService.sendCardNotice(mqList);
        }

        return Response.createSuccess();
    }

    @Override
    public Response syncRnrRelation(RnrRelationDTO bean) {
        log.info("Mg syncRnrRelation:" + JSON.toJSONString(bean));

        if (CollectionUtils.isEmpty(bean.getCardList())) {
            return Response.createError(ResponseCode.ICCID_NOT_NULL.getMsg(), ResponseCode.ICCID_NOT_NULL.getCode());
        }

        MgRnrInfoDTO info = bean.getInfo();
        RnrOrderDTO order = bean.getOrder();

        //组装卡信息
        List<MgRnrCardInfoPO> cardPOList = new ArrayList<>();

        for (MgRnrCardInfoDTO cardDTO : bean.getCardList()) {
            //设置卡信息
            cardDTO.setUuid(CuscStringUtils.generateUuid());
            cardDTO.setRnrId(info.getUuid());
            cardDTO.setTenantNo(info.getTenantNo());
            cardDTO.setRoutingKey(info.getRoutingKey());
            cardDTO.setNoticeStatus(NoticeStatusEnum.NONEED.getCode());
            cardDTO.setOrderId(order.getUuid());
            cardDTO.setRnrStatus(info.getRnrStatus());
            MgRnrCardInfoPO cardInfoPO = MgRnrCardInfoConverter.INSTANCE.dtoToPo(cardDTO);
            cardPOList.add(cardInfoPO);
        }

        //保存实名表数据
        rnrInfoService.insert(info);

        //保存工单信息
        rnrOrderService.insertOrder(order);

        //保存卡信息
        rnrCardInfoService.addRnrCardBatch(cardPOList);

        //保存tag信息
        if (CollectionUtils.isNotEmpty(bean.getRnrTagList())) {
            rnrTagService.addRnrTagFromRnrInfo(info, bean.getRnrTagList());
        }

        //保存联系人信息
        if (CollectionUtils.isNotEmpty(bean.getRnrLiaisonList())) {
            rnrLiaisonInfoService.addRnrLiaisonFromRnrInfo(info, bean.getRnrLiaisonList());
        }

        //保存公司信息
        if (bean.getCompanyInfo() != null) {
            mgRnrCompanyInfoService.insert(bean.getCompanyInfo());
        }

        //保存图片信息
        rnrFileService.addRnrFileFromInfo(info, bean.getRnrFileList());


        //非委托人模式，自然人新车和二手车成功需要创建用户
        if (!bean.getOrder().getSendWorkOrder() && RnrOrderUtil.needCreateUser(bean.getInfo(), bean.getOrder()) && CuscStringUtils.isEmpty(bean.getInfo().getUserId()) && !RnrOrderSourceEnum.H5_CUSTOMER.getCode().equals(bean.getOrder().getOrderSource())) {
            ciamService.createUser(bean.getInfo());
        }

        return Response.createSuccess("同步实名信息成功");
    }

    @Override
    public void lockRnrIccid(String tenantNo, String iccid, String value) {
        boolean lockStatus = true;
        try {
            lockStatus = cacheFactory.getLockService()
                    .setNx(RedisConstant.RNR_LOCK + tenantNo + ":" + iccid, value, RedisConstant.RNR_LOCK_TIME);
        } catch (CacheException e) {
            log.error("RnrRelationServiceImpl.lockRnrIccid 加锁失败，tenantNo: " + tenantNo + ",iccid: " + iccid +
                    ",value: " + value + ". ", e);
        }

        //锁定失败
        if (!lockStatus) {
            throw new CuscUserException(RnrResponseCode.WAIT_CHECK.getCode(),
                    RnrResponseCode.WAIT_CHECK.getDescription());
        }
    }

    @Override
    public void lockRnrIccidList(String tenantNo, List<String> iccidList, String value) {
        for (String iccid : iccidList) {
            lockRnrIccid(tenantNo, iccid, value);
        }
    }

    @Override
    public void unLockRnrIccid(String tenantNo, String iccid) {
        try {
            cacheFactory.getLockService().delete(RedisConstant.RNR_LOCK + tenantNo + ":" + iccid);
        } catch (CacheException e) {
            log.error("RnrRelationServiceImpl.unLockRnrIccid 解锁失败，tenantNo: " + tenantNo + ",iccid: " + iccid + ". ",
                    e);
        }
    }

    @Override
    public void unLockRnrIccidList(String tenantNo, List<String> iccidList) {
        for (String iccid : iccidList) {
            unLockRnrIccid(tenantNo, iccid);
        }
    }

    @Override
    public void checkIccidStatus(String tenantNo, List<String> iccidList) {
        MgRnrCardInfoDTO checkCardDTO = new MgRnrCardInfoDTO();
        checkCardDTO.setTenantNo(tenantNo);
        checkCardDTO.setIccidList(iccidList);
        boolean ret = rnrCardInfoService.checkIccidExists(checkCardDTO);
        if (ret) {
            // 如果是待审核或待人工审核则抛出，已提交申请异常
            throw new CuscUserException(RnrResponseCode.WAIT_CHECK.getCode(),
                    RnrResponseCode.WAIT_CHECK.getDescription());
        }

    }

    @Override
    public Response<RnrRelationDTO> getRnrRelation(MgRnrCardInfoDTO bean) {
        RnrRelationDTO rnrRelationDTO = new RnrRelationDTO();
        MgRnrCardInfoDTO mgRnrCardInfoDTO = new MgRnrCardInfoDTO();
        mgRnrCardInfoDTO.setIccid(bean.getIccid());
        mgRnrCardInfoDTO.setTenantNo(bean.getTenantNo());
        mgRnrCardInfoDTO = rnrCardInfoService.getNewRnrCardByIccid(mgRnrCardInfoDTO);
        if (mgRnrCardInfoDTO == null) {
            return Response.createError("未查询到已实名的卡信息");
        }

        rnrRelationDTO.setCardList(Arrays.asList(mgRnrCardInfoDTO));

        //查询实名信息
        MgRnrInfoDTO mgRnrInfoDTO = new MgRnrInfoDTO();
        mgRnrInfoDTO.setUuid(mgRnrCardInfoDTO.getRnrId());
        MgRnrInfoDTO byUuid = rnrInfoService.getByUuid(mgRnrInfoDTO);
        rnrRelationDTO.setInfo(byUuid);
        if (byUuid != null && byUuid.getIsCompany() != 0 && CuscStringUtils.isNotEmpty(byUuid.getRnrCompanyId())) {
            MgRnrCompanyInfoDTO rnrCompanyInfoDTO = new MgRnrCompanyInfoDTO();
            rnrCompanyInfoDTO.setUuid(byUuid.getRnrCompanyId());
            MgRnrCompanyInfoDTO mgRnrCompanyInfoDTO = rnrCompanyInfoService.getByUuid(rnrCompanyInfoDTO);
            rnrRelationDTO.setCompanyInfo(mgRnrCompanyInfoDTO);
        }

        //查询联系人信息
        MgRnrLiaisonInfoDTO rnrLiaisonInfoDTO = new MgRnrLiaisonInfoDTO();
        rnrLiaisonInfoDTO.setRnrId(mgRnrCardInfoDTO.getRnrId());
        rnrLiaisonInfoDTO.setTenantNo(mgRnrCardInfoDTO.getTenantNo());
        List<MgRnrLiaisonInfoDTO> liaisonInfoList = rnrLiaisonInfoService.queryByList(rnrLiaisonInfoDTO);
        rnrRelationDTO.setRnrLiaisonList(liaisonInfoList);

        //查询文件信息
        MgRnrFileDTO fileDTO = new MgRnrFileDTO();
        fileDTO.setRnrId(mgRnrCardInfoDTO.getRnrId());
        fileDTO.setTenantNo(bean.getTenantNo());
        rnrRelationDTO.setRnrFileList(rnrFileService.getByRnrid(fileDTO));

        return Response.createSuccess(rnrRelationDTO);
    }



    //----------------私有方法区-----------------

    /**
     * 参数验证是否有效
     * 只能使用Spring Validate做基础校验，NotNull,NotEmpty,Max,Min等
     *
     * @param bean
     * @return
     */
    private Response checkDTOValid(RnrRelationDTO bean) {

        Set<ConstraintViolation<RnrRelationDTO>> constraintViolations =
                SpringValidationUtil.groupVerificationParameters(bean, SaveRelation.class);

        if (!constraintViolations.isEmpty()) {
            String message = constraintViolations.stream().findFirst()
                    .get().getMessage();
            return Response.createError(message, ResponseCode.INVALID_DATA.getCode());
        }

        return Response.createSuccess();
    }

}
