package com.cusc.nirvana.user.rnr.mg.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.cache.CacheFactory;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.rnr.mg.service.NoticeConfigService;
import com.cusc.nirvana.user.rnr.mg.service.TimingSendService;
import com.cusc.nirvana.user.rnr.mg.util.RnrMgRestTemplateUtils;
import com.cusc.nirvana.user.rnr.notice.dto.RnrNoticeConfigDTO;
import com.cusc.nirvana.user.rnr.notice.dto.RnrNoticeContentDTO;
import com.cusc.nirvana.user.rnr.notice.service.IRnrNoticeContentService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import javax.annotation.Resource;
import java.util.*;

/**
 * @className: TimingSendServiceImpl
 * @author: jk
 * @date: 2022/7/29 17:20
 * @version: 1.0
 **/
@Service
@Slf4j
public class TimingSendServiceImpl implements TimingSendService {

    @Resource
    private CacheFactory cacheFactory;

    @Autowired
    @Qualifier("thirdPartyRnr")
    private RestTemplate thirdPartyRnr;

    @Autowired
    @Qualifier("restTemplate")
    private RestTemplate restTemplate;

    @Resource
    private NoticeConfigService noticeConfigService;

    @Resource
    private IRnrNoticeContentService iRnrNoticeContentService;

    @Override
    public void sendKafka(List<RnrNoticeContentDTO> list) {

        //定义参数格式为json
        final HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON_UTF8);

        for (RnrNoticeContentDTO rnrNoticeContentDTO : list) {
            if (rnrNoticeContentDTO.getCount() <= 10) {

                try {

                    RnrNoticeConfigDTO rnrNoticeConfigDTO1 = noticeConfigService.queryNoticeConfig(rnrNoticeContentDTO.getNoticeConfigId());

                    if (null != rnrNoticeConfigDTO1) {

                        HttpEntity<Map<String, Object>> httpEntity = new HttpEntity<>(JSON.parseObject(rnrNoticeContentDTO.getNoticeContent()), headers);

                        Response ret = null;
                        //1为域名调用 2为服务名调用
                        log.info("RnrNoticeConsumer start , url = {},request = {}", rnrNoticeConfigDTO1.getUrl(), rnrNoticeContentDTO.getNoticeContent());
                        if (1 == rnrNoticeConfigDTO1.getUrlType()) {
                            ret = RnrMgRestTemplateUtils.postForObject(restTemplate, rnrNoticeConfigDTO1.getUrl(), httpEntity, Response.class);
                        } else {
                            ret = RnrMgRestTemplateUtils.postForObject(thirdPartyRnr, rnrNoticeConfigDTO1.getUrl(), httpEntity, Response.class);
                        }
                        log.info("RnrNoticeConsumer end , url = {},request = {},response = {}", rnrNoticeConfigDTO1.getUrl(), rnrNoticeContentDTO.getNoticeContent(), JSONObject.toJSONString(ret));
                        RnrNoticeContentDTO rnrNotice = new RnrNoticeContentDTO();
                        rnrNotice.setUuid(rnrNoticeContentDTO.getUuid());
                        if (null == ret || !ret.getSuccess()) {
                            rnrNotice.setCount(rnrNoticeContentDTO.getCount() + 1);
                        }else {
                            rnrNotice.setNoticeStatus(0);
                        }
                        iRnrNoticeContentService.update(rnrNotice);
                    }
                } catch (Exception e) {
                    RnrNoticeContentDTO rnrNotice = new RnrNoticeContentDTO();
                    rnrNotice.setUuid(rnrNoticeContentDTO.getUuid());
                    rnrNotice.setCount(rnrNoticeContentDTO.getCount() + 1);
                    iRnrNoticeContentService.update(rnrNotice);
                }

            }
        }

    }


    public static void main(String[] args) {
        String a = "{iccid=89860900000000000005, orderId=5ea7328ca34442798fe60ade4db6b6a3, oldIccid=null, tenantNo=1, resultCode=3, vin=CUSC0000001000004, manufacturerOrgId=aab1ae4c00d049e99d144d52b945ec5d, orgId=null, rnrBizzType=1}";
        Map<String, Object> b = mapStringToMap(a);
        System.out.println(JSONObject.toJSONString(b));

        Map<String, Object>  maps =  JSON.parseObject(JSONObject.toJSONString(b),Map.class);

        System.out.println(maps);
    }

    private static Map<String, Object> mapStringToMap(String str) {
        str = str.substring(1, str.length() - 1);
        String[] strs = str.split(",");
        Map<String, Object> map = new HashMap<>();
        for (String string : strs) {
            String key = string.split("=")[0].trim();
            String value = string.split("=")[1];
            map.put(key, value);
        }
        return map;
    }


}
