package com.cusc.nirvana.user.rnr.mg.util;

import net.coobird.thumbnailator.Thumbnails;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileItemFactory;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.MediaType;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.commons.CommonsMultipartFile;
import sun.misc.BASE64Decoder;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.*;
import java.util.UUID;

public class Base64ConvertUitl {
    private static final Logger logger = LoggerFactory.getLogger(Base64ConvertUitl.class);

    private static final String FILE_EXT = "jpg";
    private static final String VEDIO_EXT = "mp4";

    /**
     * 将文件转成base64 字符串
     *
     * @return
     * @throws Exception
     */

    public static String encodeBase64File(String path) throws Exception {
        File file = new File(path);
        FileInputStream inputFile = new FileInputStream(file);
        byte[] buffer = new byte[(int) file.length()];
        inputFile.read(buffer);
        inputFile.close();
        byte[] encode = new Base64().encode(buffer);
        return encode.toString();

    }

    /**
     * 将base64字符解码保存文件
     *
     * @param base64
     */
    public static MultipartFile decoderBase64File(String base64, String tmpDir) {
        String sourceFile = tmpDir + UUID.randomUUID() + "." + FILE_EXT;
        byte[] content = Base64.decodeBase64(base64);
        writeBytesToFile(content, sourceFile);
        File file = new File(sourceFile);
        FileItem fileItem = createFileItem(file);
        MultipartFile multipartFile = new CommonsMultipartFile(fileItem);
        if (file != null) {
            try {
                file.delete();//删除本地保存的文件
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return multipartFile;
    }

    /**
     * 将base64字符解码保存文件
     * @param base64
     * @param tmpDir
     * @return
     */
    public static MultipartFile decoderBase64File2Video(String base64, String tmpDir) {
        String sourceFile = tmpDir + UUID.randomUUID() + "." + VEDIO_EXT;
        byte[] content = Base64.decodeBase64(base64);
        writeBytesToFile(content, sourceFile);
        File file = new File(sourceFile);
        FileItem fileItem = createFileItem(file);
        MultipartFile multipartFile = new CommonsMultipartFile(fileItem);
        if (file != null) {
            try {
                file.delete();//删除本地保存的文件
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return multipartFile;
    }



    public static FileItem createFileItem(File file) {
        FileItemFactory factory = new DiskFileItemFactory(16, null);
        String textFieldName = "file";
        FileItem item = factory.createItem(textFieldName, MediaType.MULTIPART_FORM_DATA_VALUE, true,
                file.getName());
        int bytesRead = 0;
        byte[] buffer = new byte[8192];
        try {
            FileInputStream fis = new FileInputStream(file);
            OutputStream os = item.getOutputStream();
            while ((bytesRead = fis.read(buffer, 0, 8192)) != -1) {
                os.write(buffer, 0, bytesRead);
            }
            os.close();
            fis.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return item;
    }

    public static boolean writeBytesToFile(byte[] content, String sourceFile) {
        try (ByteArrayInputStream bis = new ByteArrayInputStream(content);
             OutputStream out = new FileOutputStream(sourceFile)) {
            byte[] data = new byte[1024];
            int size = 0;
            while ((size = bis.read(data)) != -1) {
                out.write(data, 0, size);
            }
            out.flush();
            return true;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return false;
    }

    /**
     * 将base64转成视频文件(mp4格式)
     *
     * @param base64
     * @param videoFilePath
     */
    public static void decoderBase64ToVideo(String base64, String videoFilePath) {
        String[] split = base64.split(",");
        if (split.length > 1) {
            base64 = String.valueOf(split[1]);
        }
        //base解密
        byte[] videoByte = new byte[0];
        FileOutputStream fos = null;
        try {
            videoByte = new BASE64Decoder().decodeBuffer(base64);
            File videoFile = new File(videoFilePath);
            //输入视频文件
            fos = new FileOutputStream(videoFile);
            fos.write(videoByte, 0, videoByte.length);
            fos.flush();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (fos != null) {
                try {
                    fos.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    /**
     * @param base64Str
     * @return
     */
    public static BufferedImage base64Str2BufferedImage(String base64Str) {
        BufferedImage image = null;
        try {
            InputStream inputStream = baseToInputStream(base64Str);
            image = ImageIO.read(inputStream);
        } catch (IOException e) {
            logger.error("base642BufferedImage error={}", e);
        }
        return image;

    }

    /**
     * @param base64Str
     * @return
     */
    private static InputStream baseToInputStream(String base64Str) {
        ByteArrayInputStream stream = null;
        try {
            BASE64Decoder decoder = new BASE64Decoder();
            byte[] bytes1 = decoder.decodeBuffer(base64Str);
            stream = new ByteArrayInputStream(bytes1);
        } catch (Exception e) {
            logger.error("baseToInputStream error={}", e);
        } finally {
            if (stream != null) {
                try {
                    stream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return stream;
    }

    /**
     * @param bufferedImage
     * @return
     */
    public static String imageToBase64(BufferedImage bufferedImage) {
        Base64 encoder = new Base64();
        ByteArrayOutputStream baos = null;
        try {
            baos = new ByteArrayOutputStream();
            ImageIO.write(bufferedImage, FILE_EXT, baos);
            String base64 = new String(encoder.encode((baos.toByteArray())));
            return base64;
        } catch (IOException e) {
            logger.error("imageToBase64 error={}", e);
        } finally {
            if (baos != null) {
                try {
                    baos.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return null;
    }

    /**
     * @param base64Img
     * @param reSize    (例如32K 只需要传32 即可)
     * @return
     */
    public static String resizeImage(String base64Img, int reSize) {
        int reSizeK = reSize * 1024;
        try {
            BufferedImage src = base64Str2BufferedImage(base64Img);
            BufferedImage output = Thumbnails.of(src).size(src.getWidth() / 3, src.getHeight() / 3).asBufferedImage();
            String base64 = imageToBase64(output);
            if (base64.length() > reSizeK) {
                output = Thumbnails.of(output).scale(1 / (base64.length() / reSizeK)).asBufferedImage();
                base64 = imageToBase64(output);
            }
            return base64;
        } catch (Exception e) {
            return base64Img;
        }
    }

    /**
     * 转换成大小为32K一下大照片base64
     *
     * @param base64Img
     * @return
     */
    public static String resizeImageTo32K(String base64Img) {
        boolean resizeFlag = base64Img.length() > 32 * 1024;
        try {
            while (resizeFlag) {
                BufferedImage src = base64Str2BufferedImage(base64Img);
                BufferedImage output = Thumbnails.of(src).size(src.getWidth() * 4 / 5, src.getHeight() * 4 / 5).asBufferedImage();
                base64Img = imageToBase64(output);
                resizeFlag = base64Img.length() > 32 * 1024;
            }
            return base64Img;
        } catch (Exception e) {
            return base64Img;
        }
    }

    public static String fileToBase64(File file) {
        if (file == null) {
            return null;
        }
        String base64 = null;
        FileInputStream fin = null;
        try {
            fin = new FileInputStream(file);
            byte[] buff = new byte[fin.available()];
            fin.read(buff);
            base64 = com.sun.org.apache.xerces.internal.impl.dv.util.Base64.encode(buff);
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                if (fin != null) {
                    fin.close();
                }
                if (file != null && file.exists()) {
                    file.delete();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return base64;
    }

    public static void main(String[] args) throws Exception {
        String base64 = fileToBase64(new File("E:\\1.jpg"));
        System.out.println("原始大小=" + (base64.length() / 1024));
        String resizeBase64 = resizeImageTo32K(base64);
        System.out.println("压缩大小=" + (resizeBase64.length() / 1024));
        System.out.println(resizeBase64);
    }
}
