package com.cusc.nirvana.user.rnr.notice.kafka;

import com.alibaba.fastjson.JSON;
import com.cusc.nirvana.user.rnr.mg.constants.RnrMgMqConstant;
import com.cusc.nirvana.user.rnr.mg.dto.MgCardNoticeDTO;
import com.cusc.nirvana.user.rnr.notice.service.NoticeHandler;
import com.cusc.nirvana.user.rnr.notice.service.NoticeHandlerContext;
import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.stereotype.Service;

import java.util.Objects;
import java.util.Optional;

/**
 * 绑定、解绑 队列消费
 *
 * @author huzl
 */
@Slf4j
@Service
public class KfKRnrNoticeConsumer {

    @Autowired
    NoticeHandlerContext handlerContext;


    @KafkaListener(id = RnrMgMqConstant.RNR_CARD_OPERATION_GROUP, topics = {RnrMgMqConstant.RNR_CARD_OPERATION_TOPIC})
    public void onMessage(ConsumerRecord<?, ?> record) {

        Optional<?> kafkaMessage = Optional.ofNullable(record.value());
        MgCardNoticeDTO bean = null;
        if (kafkaMessage.isPresent()) {
            Object message = kafkaMessage.get();
            bean = JSON.parseObject(message.toString(), MgCardNoticeDTO.class);
        }
        log.info("RnrNoticeConsumer onMessage: {}", JSON.toJSONString(bean));
        if (Objects.isNull(bean)) {
            return;
        }

        for (NoticeHandler noticeHandler : handlerContext.getHandlerList()) {
            noticeHandler.handleNotice(bean);
        }
    }

}
