package com.cusc.nirvana.user.rnr.notice.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.cache.CacheFactory;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.rnr.common.constants.RedisConstant;
import com.cusc.nirvana.user.rnr.mg.dto.MgCardNoticeDTO;
import com.cusc.nirvana.user.rnr.mg.util.RnrMgRestTemplateUtils;
import com.cusc.nirvana.user.rnr.notice.dto.RnrNoticeConfigDTO;
import com.cusc.nirvana.user.rnr.notice.dto.RnrNoticeContentDTO;
import com.cusc.nirvana.user.rnr.notice.dto.RnrNoticeContentLogDTO;
import com.cusc.nirvana.user.rnr.notice.service.IRnrNoticeConfigService;
import com.cusc.nirvana.user.rnr.notice.service.IRnrNoticeContentLogService;
import com.cusc.nirvana.user.rnr.notice.service.IRnrNoticeContentService;
import com.cusc.nirvana.user.rnr.notice.service.NoticeHandler;
import com.cusc.nirvana.user.util.CuscStringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author yubo
 * @since 2022-06-16 18:12
 */
@Service("noticeContentHandler")
@Slf4j
public class NoticeContentHandler implements NoticeHandler {

    @Autowired
    private IRnrNoticeConfigService rnrNoticeConfigService;
    @Autowired
    private IRnrNoticeContentLogService rnrNoticeContentLogService;
    @Autowired
    private IRnrNoticeContentService rnrNoticeContentService;
    @Autowired
    @Qualifier("thirdPartyRnr")
    private RestTemplate thirdPartyRnr;

        @Autowired
        @Qualifier("restTemplate")
        private RestTemplate restTemplate;

    @Override
    public void handleNotice(MgCardNoticeDTO bean) {
        log.warn("handleNotice入参{}",JSONObject.toJSONString(bean));
        try {
            RnrNoticeConfigDTO paramDTO = new RnrNoticeConfigDTO();
            //通过项目编号查询单条数据
            paramDTO.setNoticeSubject(bean.getManufacturerOrgId());
            paramDTO.setTenantNo(bean.getTenantNo());
            List<RnrNoticeConfigDTO> rnrNoticeConfigList = rnrNoticeConfigService.queryByNoticeSubject(paramDTO);
            if (CollectionUtils.isEmpty(rnrNoticeConfigList)) {
                log.info("RnrNoticeConsumer notice config is null , request: {}", JSON.toJSONString(bean));
                return;
            }
            for (RnrNoticeConfigDTO rnrNoticeConfigDTO : rnrNoticeConfigList) {
                String url = rnrNoticeConfigDTO.getUrl();
                Integer urlType = rnrNoticeConfigDTO.getUrlType();

                //定义参数格式为json
                final HttpHeaders headers = new HttpHeaders();
                headers.setContentType(MediaType.APPLICATION_JSON_UTF8);

                //调通知接口时传的参数
                Map<String, Object> parameters = new HashMap<>();
                parameters.put("vin", bean.getVin());
                parameters.put("iccid", bean.getIccid());
                parameters.put("oldIccid", bean.getOldIccid());
                parameters.put("orderId", bean.getOrderId());
                parameters.put("orgId", bean.getOrgId());
                parameters.put("manufacturerOrgId", bean.getManufacturerOrgId());

                parameters.put("rnrBizzType", bean.getRnrBizzType());
                parameters.put("resultCode", bean.getResultCode());
                parameters.put("tenantNo", bean.getTenantNo());

                HttpEntity<Map<String, Object>> httpEntity = new HttpEntity<>(parameters, headers);
                Date requestTime = new Date();
                Response ret = null;
                //1为域名调用 2为服务名调用
                log.info("RnrNoticeConsumer start , url = {},request = {}", url, JSONObject.toJSONString(parameters));
                if (1 == urlType) {
                    ret = RnrMgRestTemplateUtils.postForObject(restTemplate, url, httpEntity, Response.class);
                } else {
                    ret = RnrMgRestTemplateUtils.postForObject(thirdPartyRnr, url, httpEntity, Response.class);
                }
                log.info("RnrNoticeConsumer end , url = {},request = {},response = {}", url, JSONObject.toJSONString(parameters), JSONObject.toJSONString(ret));
                Date responseTime = new Date();
                //0-成功，1-失败
                Integer noticeStatus = null;
                if (Objects.nonNull(ret) && ret.getSuccess()) {
                    noticeStatus = 0;
                } else {
                    noticeStatus = 1;
                }

                //插入rnr_notice_content表
                RnrNoticeContentDTO rnrNoticeContentDTO = new RnrNoticeContentDTO();
                rnrNoticeContentDTO.setUuid(CuscStringUtils.generateUuid());
                rnrNoticeContentDTO.setNoticeConfigId(rnrNoticeConfigDTO.getUuid());
                rnrNoticeContentDTO.setNoticeSubject("");
                rnrNoticeContentDTO.setRnrBizzType(bean.getRnrBizzType());
                rnrNoticeContentDTO.setNoticeContent(JSON.toJSONString(parameters));
                rnrNoticeContentDTO.setNoticeStatus(noticeStatus);
                rnrNoticeContentDTO.setCount(1);
                rnrNoticeContentDTO.setCreator(bean.getOperator());
                rnrNoticeContentService.insert(rnrNoticeContentDTO);

                //插入rnr_notice_content_log表
                RnrNoticeContentLogDTO rnrNoticeContentLogDTO = new RnrNoticeContentLogDTO();
                rnrNoticeContentLogDTO.setUuid(CuscStringUtils.generateUuid());
                rnrNoticeContentLogDTO.setRnrBizzType(bean.getRnrBizzType());
                rnrNoticeContentLogDTO.setNoticeContentId(rnrNoticeContentDTO.getUuid());
                rnrNoticeContentLogDTO.setRequestTime(requestTime);
                rnrNoticeContentLogDTO.setResponseTime(responseTime);
                if (Objects.nonNull(ret)) {
                    rnrNoticeContentLogDTO.setResponseMsg(ret.getMsg());
                }
                //0-成功，1-失败
                rnrNoticeContentLogDTO.setIsSuccess(noticeStatus);
                rnrNoticeContentLogDTO.setCreator(bean.getOperator());
                rnrNoticeContentLogService.insert(rnrNoticeContentLogDTO);
            }

        } catch (Exception e) {
            log.error("RnrNoticeConsumer 消费异常: data = {}", JSONObject.toJSONString(bean),e);
        }
    }
}
