package com.cusc.nirvana.user.rnr.notice.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cache.annotation.tag.Cache;
import com.cusc.nirvana.common.result.PageResult;
import com.cusc.nirvana.rds.mybatis.PageHelper;
import com.cusc.nirvana.user.rnr.common.constants.RedisConstant;
import com.cusc.nirvana.user.rnr.mg.constants.CommonDeleteEnum;
import com.cusc.nirvana.user.rnr.notice.converter.RnrNoticeConfigConverter;
import com.cusc.nirvana.user.rnr.notice.dao.RnrNoticeConfigDao;
import com.cusc.nirvana.user.rnr.notice.dao.entity.RnrNoticeConfigPO;
import com.cusc.nirvana.user.rnr.notice.dto.RnrNoticeConfigDTO;
import com.cusc.nirvana.user.rnr.notice.service.IRnrNoticeConfigService;
import com.cusc.nirvana.user.util.CuscStringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.List;

/**
 * 实名通知配置(同一个主体和状态支持多个通知方)(RnrNoticeConfig)表服务实现类
 *
 * @author yuy336
 * @since 2022-03-31 15:04:52
 */
@Service
public class RnrNoticeConfigServiceImpl extends ServiceImpl<RnrNoticeConfigDao, RnrNoticeConfigPO>
        implements IRnrNoticeConfigService {

    /**
     * 通过UUID查询单条数据
     *
     * @param bean
     * @return 实例对象
     */
    @Override
    public RnrNoticeConfigDTO getByUuid(RnrNoticeConfigDTO bean) {
        RnrNoticeConfigPO record = this.getPoByUuid(bean.getUuid());
        return RnrNoticeConfigConverter.INSTANCE.poToDto(record);
    }


    @Override
    public List<RnrNoticeConfigDTO> queryByNoticeSubject(RnrNoticeConfigDTO bean) {
        List<RnrNoticeConfigPO> record = this.getPoByNoticeSubject(bean);
        return RnrNoticeConfigConverter.INSTANCE.poListToDtoList(record);
    }

    /**
     * 通过查询条件查询集合数据
     *
     * @param bean
     * @return 集合对象
     */
    @Override
    public List<RnrNoticeConfigDTO> queryByList(RnrNoticeConfigDTO bean) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());
        queryWrapper.orderByDesc("create_time");
        List<RnrNoticeConfigPO> record = this.list(queryWrapper);
        return RnrNoticeConfigConverter.INSTANCE.poListToDtoList(record);
    }

    /**
     * 分页查询
     *
     * @param bean 筛选条件
     * @return 查询结果
     */
    @Override
    public PageResult<RnrNoticeConfigDTO> queryByPage(RnrNoticeConfigDTO bean) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.orderByDesc("create_time");
        queryWrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());

        Page<RnrNoticeConfigPO> page =
                this.page(new Page<>(bean.getCurrPage(), bean.getPageSize()), queryWrapper);
        return PageHelper.convert(page, RnrNoticeConfigDTO.class);
    }

    /**
     * 新增数据
     *
     * @param bean 实例对象
     * @return 实例对象
     */
    @Override
    @Transactional
    @Cache(keyPrefix = RedisConstant.RNR_NOTICE_CONFIG, key = "#bean.noticeSubject+\"_\"+#bean.noticeType", level1 =
            "redis", action = 0)
    public RnrNoticeConfigDTO insert(RnrNoticeConfigDTO bean) {
        RnrNoticeConfigPO rnrNoticeConfigPO = RnrNoticeConfigConverter.INSTANCE.dtoToPo(bean);
        rnrNoticeConfigPO.setUuid(CuscStringUtils.generateUuid());
        this.save(rnrNoticeConfigPO);
        bean.setUuid(rnrNoticeConfigPO.getUuid());
        return bean;
    }

    /**
     * 修改数据
     *
     * @param bean 实例对象
     * @return 实例对象
     */
    @Override
    @Transactional
    @Cache(keyPrefix = RedisConstant.RNR_NOTICE_CONFIG, key = "#bean.noticeSubject+\"_\"+#bean.noticeType", level1 =
            "redis", action = 0)
    public RnrNoticeConfigDTO update(RnrNoticeConfigDTO bean) {
        RnrNoticeConfigPO rnrNoticeConfigPO = this.getPoByUuid(bean.getUuid());
        if (rnrNoticeConfigPO == null) {
            return null;
        }
        RnrNoticeConfigPO tmpBean = RnrNoticeConfigConverter.INSTANCE.dtoToPo(bean);
        tmpBean.setId(rnrNoticeConfigPO.getId());
        this.updateById(tmpBean);
        return bean;
    }

    /**
     * 通过主键删除数据
     *
     * @param bean 实例对象
     * @return 是否成功
     */
    @Override
    @Transactional
    @Cache(keyPrefix = RedisConstant.RNR_NOTICE_CONFIG, key = "#bean.noticeSubject+\"_\"+#bean.noticeType", level1 =
            "redis", action = 0)
    public boolean deleteById(RnrNoticeConfigDTO bean) {
        RnrNoticeConfigPO rnrNoticeConfigPO = this.getPoByUuid(bean.getUuid());
        if (rnrNoticeConfigPO == null) {
            return false;
        }
        RnrNoticeConfigPO tmpBean = new RnrNoticeConfigPO();
        tmpBean.setId(rnrNoticeConfigPO.getId());
        tmpBean.setIsDelete(CommonDeleteEnum.DELETED.getCode());
        return this.updateById(tmpBean);
    }

    /**
     * 通过业务主体和通知类型查询通知配置（支持缓存）
     *
     * @param bean
     * @return 集合对象
     */
    @Override
    @Cache(keyPrefix = RedisConstant.RNR_NOTICE_CONFIG, key = "#bean.noticeSubject+\"_\"+#bean.rnrBizzType", level1 =
            "redis")
    public List<RnrNoticeConfigDTO> queryBySubjectAndTypeCache(RnrNoticeConfigDTO bean) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("notice_subject", bean.getNoticeSubject());
        queryWrapper.eq("notice_type", bean.getRnrBizzType());
        try {
            List<RnrNoticeConfigPO> configListPO = this.list(queryWrapper);
            if (CollectionUtils.isEmpty(configListPO)) {
                return null;
            }
            return RnrNoticeConfigConverter.INSTANCE.poListToDtoList(configListPO);
        } catch (Exception e) {
            log.error("queryBySubjectAndTypeCache  ", e);
        }
        return null;

    }

    //----------------------私有方法区----------------------

    /**
     * 通过UUID查询单条数据
     *
     * @param uuid
     * @return 实例对象
     */
    private RnrNoticeConfigPO getPoByUuid(String uuid) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("uuid", uuid);
        queryWrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());
        return this.getOne(queryWrapper);
    }

    /**
     * 通过项目编号查询单条数据
     *
     * @param bean
     * @return 实例对象
     */
    private List<RnrNoticeConfigPO> getPoByNoticeSubject(RnrNoticeConfigDTO bean) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("notice_subject", bean.getNoticeSubject());
        queryWrapper.eq("tenant_no", bean.getTenantNo());
        queryWrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());
        return this.list(queryWrapper);
    }
}
