package com.cusc.nirvana.user.rnr.notice.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cusc.nirvana.common.result.PageResult;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.rds.mybatis.PageHelper;
import com.cusc.nirvana.user.rnr.mg.constants.CommonDeleteEnum;
import com.cusc.nirvana.user.rnr.mg.constants.CommonYesOrNoEnum;
import com.cusc.nirvana.user.rnr.mg.dao.entity.SimFileHistoryPO;
import com.cusc.nirvana.user.rnr.mg.dto.SimFileHistoryDTO;
import com.cusc.nirvana.user.rnr.mg.dto.SimFileHistoryQueryDTO;
import com.cusc.nirvana.user.rnr.mg.util.Md5CaculateUtil;
import com.cusc.nirvana.user.rnr.notice.constants.NoticeUrlTypeEnum;
import com.cusc.nirvana.user.rnr.notice.converter.RnrNoticeContentConverter;
import com.cusc.nirvana.user.rnr.notice.converter.ThirdPartyNoticeConverter;
import com.cusc.nirvana.user.rnr.notice.dao.RnrNoticeContentDao;
import com.cusc.nirvana.user.rnr.notice.dao.entity.RnrNoticeContentPO;
import com.cusc.nirvana.user.rnr.notice.dto.RnrNoticeConfigDTO;
import com.cusc.nirvana.user.rnr.notice.dto.RnrNoticeContentDTO;
import com.cusc.nirvana.user.rnr.notice.dto.RnrNoticeContentLogDTO;
import com.cusc.nirvana.user.rnr.notice.dto.RnrNoticeReqDTO;
import com.cusc.nirvana.user.rnr.notice.dto.ThirdPartyNoticeReqDTO;
import com.cusc.nirvana.user.rnr.notice.service.IRnrNoticeConfigService;
import com.cusc.nirvana.user.rnr.notice.service.IRnrNoticeContentLogService;
import com.cusc.nirvana.user.rnr.notice.service.IRnrNoticeContentService;
import com.cusc.nirvana.user.util.CuscStringUtils;
import com.cusc.nirvana.user.util.DateUtils;
import org.apache.skywalking.apm.toolkit.trace.TraceContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.client.RestTemplate;

import java.util.Date;
import java.util.List;

/**
 * 实名通知内容(RnrNoticeContent)表服务实现类
 *
 * @author yuy336
 * @since 2022-03-31 15:05:11
 */
@Service
public class RnrNoticeContentServiceImpl extends ServiceImpl<RnrNoticeContentDao, RnrNoticeContentPO>
        implements IRnrNoticeContentService {

    @Autowired
    private IRnrNoticeContentLogService rnrNoticeContentLogService;

    @Autowired
    private IRnrNoticeConfigService rnrNoticeConfigService;

    @Autowired
    @Qualifier("restTemplate")
    public RestTemplate restTemplate;

    @Autowired
    @Qualifier("thirdPartyRnr")
    public RestTemplate thirdPartyRnr;

    /**
     * 通过UUID查询单条数据
     *
     * @param bean
     * @return 实例对象
     */
    @Override
    public RnrNoticeContentDTO getByUuid(RnrNoticeContentDTO bean) {
        RnrNoticeContentPO record = this.getPoByUuid(bean.getUuid());
        return RnrNoticeContentConverter.INSTANCE.poToDto(record);
    }

    /**
     * 通过查询条件查询集合数据
     *
     * @param bean
     * @return 集合对象
     */
    @Override
    public List<RnrNoticeContentDTO> queryByList(RnrNoticeContentDTO bean) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("notice_status", 1);
        queryWrapper.orderByDesc("create_time");
        List<RnrNoticeContentPO> record = this.list(queryWrapper);
        return RnrNoticeContentConverter.INSTANCE.poListToDtoList(record);
    }

    @Override
    public PageResult<RnrNoticeContentDTO> queryByPage(RnrNoticeContentDTO queryDTO) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("notice_status", 1);
        queryWrapper.lt("count",10);
        queryWrapper.orderByDesc("create_time");

        Page<RnrNoticeContentPO> historyPOPage = this.page(new Page<>(queryDTO.getCurrPage(), queryDTO.getPageSize()), queryWrapper);
        return PageHelper.convert(historyPOPage, RnrNoticeContentDTO.class);
    }

    /**
     * 新增数据
     *
     * @param bean 实例对象
     * @return 实例对象
     */
    @Override
    @Transactional
    public RnrNoticeContentDTO insert(RnrNoticeContentDTO bean) {
        RnrNoticeContentPO rnrNoticeContentPO = RnrNoticeContentConverter.INSTANCE.dtoToPo(bean);
        if (StringUtils.isEmpty(rnrNoticeContentPO.getUuid())) {
            rnrNoticeContentPO.setUuid(CuscStringUtils.generateUuid());
        }
        this.save(rnrNoticeContentPO);
        bean.setUuid(rnrNoticeContentPO.getUuid());
        return bean;
    }

    /**
     * 修改数据
     *
     * @param bean 实例对象
     * @return 实例对象
     */
    @Override
    @Transactional
    public RnrNoticeContentDTO update(RnrNoticeContentDTO bean) {
        RnrNoticeContentPO rnrNoticeContentPO = this.getPoByUuid(bean.getUuid());
        if (rnrNoticeContentPO == null) {
            return null;
        }
        RnrNoticeContentPO tmpBean = RnrNoticeContentConverter.INSTANCE.dtoToPo(bean);
        tmpBean.setId(rnrNoticeContentPO.getId());
        this.updateById(tmpBean);
        return bean;
    }

    /**
     * 通过主键删除数据
     *
     * @param bean 实例对象
     * @return 是否成功
     */
    @Override
    @Transactional
    public boolean deleteById(RnrNoticeContentDTO bean) {
        RnrNoticeContentPO rnrNoticeContentPO = this.getPoByUuid(bean.getUuid());
        if (rnrNoticeContentPO == null) {
            return false;
        }
        RnrNoticeContentPO tmpBean = new RnrNoticeContentPO();
        tmpBean.setId(rnrNoticeContentPO.getId());
        return this.updateById(tmpBean);
    }

    /**
     * Description: 通知三方系统
     * <br />
     * CreateDate 2022-03-31 16:19:57
     *
     * @author yuyi
     **/
    @Override
    @Transactional
    public boolean noticeThreePartySystem(RnrNoticeReqDTO bean) {
        boolean ret = false;
        // 根据主体查询通知配置
        RnrNoticeConfigDTO config = new RnrNoticeConfigDTO();
        config.setNoticeSubject(bean.getNoticeSubject());
        //config.setRnrBizzType(bean.getNoticeType());
        List<RnrNoticeConfigDTO> noticeConfigList = rnrNoticeConfigService.queryBySubjectAndTypeCache(config);
        if (CollectionUtils.isEmpty(noticeConfigList)) {
            return ret;
        }
        ThirdPartyNoticeReqDTO tpNoticeReq = ThirdPartyNoticeConverter.INSTANCE.noticeReqToThirdPartyDto(bean);
        RnrNoticeContentDTO contentDTO;
        RnrNoticeContentLogDTO logDTO;
        for (RnrNoticeConfigDTO noticeConfig : noticeConfigList) {
            contentDTO = new RnrNoticeContentDTO();
            contentDTO.setNoticeConfigId(noticeConfig.getUuid());
            contentDTO.setNoticeSubject(noticeConfig.getNoticeSubject());
            contentDTO.setRnrBizzType(noticeConfig.getRnrBizzType());
            contentDTO.setCount(1);

            logDTO = new RnrNoticeContentLogDTO();
            logDTO.setRequestTime(new Date());
            logDTO.setTraceId(TraceContext.traceId());
            //发起三方通知
            Response thirdResp = thirdPartyNotice(noticeConfig, tpNoticeReq);
            logDTO.setResponseMsg(JSON.toJSONString(thirdResp));
            logDTO.setResponseTime(new Date());
            if (thirdResp.getSuccess()) {
                logDTO.setIsSuccess(CommonYesOrNoEnum.YES.getCode());
                contentDTO.setNoticeStatus(CommonYesOrNoEnum.YES.getCode());
                ret = true;
            } else {
                logDTO.setIsSuccess(CommonYesOrNoEnum.NO.getCode());
                contentDTO.setNoticeStatus(CommonYesOrNoEnum.NO.getCode());
            }
            //记录通知内容和通知日志
            contentDTO.setNoticeContent(JSON.toJSONString(tpNoticeReq));
            contentDTO = this.insert(contentDTO);

            logDTO.setNoticeContentId(contentDTO.getUuid());
            rnrNoticeContentLogService.insert(logDTO);
        }
        return ret;
    }

    @Override
    public boolean noticeThreePartySystemRetry(RnrNoticeContentDTO bean) {
        boolean ret = false;
        // 根据主体查询通知配置
        RnrNoticeConfigDTO config = new RnrNoticeConfigDTO();
        config.setNoticeSubject(bean.getNoticeSubject());
        config.setRnrBizzType(bean.getRnrBizzType());
        List<RnrNoticeConfigDTO> noticeConfigList = rnrNoticeConfigService.queryBySubjectAndTypeCache(config);
        if (CollectionUtils.isEmpty(noticeConfigList)) {
            return ret;
        }
        ThirdPartyNoticeReqDTO tpNoticeReq = JSON.parseObject(bean.getNoticeContent(), ThirdPartyNoticeReqDTO.class);
        RnrNoticeContentDTO contentDTO;
        RnrNoticeContentLogDTO logDTO;
        config = null;
        for (RnrNoticeConfigDTO noticeConfig : noticeConfigList) {
            if (noticeConfig.getUuid().equals(bean.getNoticeConfigId())) {
                config = noticeConfig;
            }
        }
        logDTO = new RnrNoticeContentLogDTO();
        logDTO.setRequestTime(new Date());
        logDTO.setTraceId(TraceContext.traceId());

        if (config != null) {
            contentDTO = new RnrNoticeContentDTO();
            contentDTO.setUuid(bean.getUuid());
            //发起三方通知
            Response thirdResp = thirdPartyNotice(config, tpNoticeReq);
            logDTO.setResponseMsg(JSON.toJSONString(thirdResp));
            logDTO.setResponseTime(new Date());
            if (thirdResp.getSuccess()) {
                logDTO.setIsSuccess(CommonYesOrNoEnum.YES.getCode());
                contentDTO.setNoticeStatus(CommonYesOrNoEnum.YES.getCode());
                ret = true;
            } else {
                logDTO.setIsSuccess(CommonYesOrNoEnum.NO.getCode());
                //更新错误次数
                contentDTO.setCount(bean.getCount() + 1);
            }
        } else {
            logDTO.setResponseMsg("通知配置不存在");
        }
        //记录通知日志
        logDTO.setNoticeContentId(bean.getUuid());
        rnrNoticeContentLogService.insert(logDTO);
        return ret;
    }

    //----------------私有方法区--------------

    /**
     * 通过UUID查询单条数据
     *
     * @param uuid
     * @return 实例对象
     */
    private RnrNoticeContentPO getPoByUuid(String uuid) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("uuid", uuid);
//        queryWrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());
        return this.getOne(queryWrapper);
    }

    /**
     * Description: 发起三方通知
     * <br />
     * CreateDate 2022-03-31 19:36:38
     *
     * @author yuyi
     **/
    private Response thirdPartyNotice(RnrNoticeConfigDTO noticeConfig, ThirdPartyNoticeReqDTO tpNoticeReq) {
        Response ret;
        if (NoticeUrlTypeEnum.SERVER_NAME.getCode() == noticeConfig.getUrlType()) {
            //服务名调用
            ret = restTemplate.postForObject(noticeConfig.getUrl(), tpNoticeReq, Response.class);
        } else {
            //ip调用
            HttpEntity reqHttpEntity;
            if (CommonYesOrNoEnum.YES.getCode() == noticeConfig.getIsSign()) {
                //远程需要header验签
                reqHttpEntity = new HttpEntity<>(tpNoticeReq, builderHttpHeaders(noticeConfig));
            } else {
                reqHttpEntity = new HttpEntity<>(tpNoticeReq);
            }
            ret = thirdPartyRnr.postForObject(noticeConfig.getUrl(), reqHttpEntity, Response.class);
        }
        return ret;
    }

    /**
     * Description: 构建header
     * <br />
     * CreateDate 2022-03-30 10:47:35
     *
     * @author yuyi
     **/
    private HttpHeaders builderHttpHeaders(RnrNoticeConfigDTO noticeConfig) {
        HttpHeaders requestHeaders = new HttpHeaders();
        requestHeaders.setContentType(MediaType.APPLICATION_JSON_UTF8);
        String currentTimeStr = DateUtils.currentTimeStr();
        String nonce = CuscStringUtils.generateUuid();
        requestHeaders.set("appid", noticeConfig.getAppId());
        requestHeaders.set("timestamp", currentTimeStr);
        requestHeaders.set("serialNumber", "");
        StringBuffer sbr = new StringBuffer();
        sbr.append("appid=");
        sbr.append(noticeConfig.getAppId());
        sbr.append("&timestamp=");
        sbr.append(currentTimeStr);
        sbr.append("&serialNumber=");
        sbr.append(nonce);
        sbr.append("&appkey=");
        sbr.append(noticeConfig.getAppKey());
        requestHeaders.set("sign", Md5CaculateUtil.MD5(sbr.toString()));
        return requestHeaders;
    }
}
