package com.cusc.nirvana.user.rnr.workorder.handler.impl;

import com.alibaba.fastjson.JSONObject;
import com.cusc.nirvana.common.loader.CollectionUtils;
import com.cusc.nirvana.user.rnr.mg.constants.RnrOrderStatusEnum;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrCardInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgVehicleCompanyDTO;
import com.cusc.nirvana.user.rnr.mg.dto.OrgSimRelDTO;
import com.cusc.nirvana.user.rnr.mg.dto.RnrOrderDTO;
import com.cusc.nirvana.user.rnr.mg.service.*;
import com.cusc.nirvana.user.rnr.workorder.dto.WorkOrderReviewCallBackRequestDTO;
import com.cusc.nirvana.user.rnr.workorder.handler.WorkOrderCallBackHandler;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * Description: 车企实名成功工单回调
 * <br />
 * CreateDate 2022-05-04 14:27:32
 *
 * @author yuyi
 **/
@Service("manufacturerRnrCallBackHandler")
@Slf4j
public class ManufacturerRnrCallBackHandler implements WorkOrderCallBackHandler {

    @Autowired
    IRnrOrderService orderService;

    @Autowired
    IMgRnrCardInfoService cardInfoService;

    @Autowired
    @Lazy
    IMgRnrCardNoticeService cardNoticeService;

    @Autowired
    IMgRnrInfoService mgRnrInfoService;

    @Autowired
    IMgVehicleCompanyService mgVehicleCompanyService;

    @Resource
	private IRnrResultProcessService processService;

    @Resource
    private IOrgSimRelService iOrgSimRelService;


    @Override
    public void claimAssignWorkOrder(String workNo) {
        //todo
    }

    @Override
    public void reviewCallBack(WorkOrderReviewCallBackRequestDTO dto) {
        String orderId = dto.getBusinessObjectId();
        RnrOrderStatusEnum statusEnum =
                StringUtils.equalsIgnoreCase(dto.getAuditResult(), "1") ? RnrOrderStatusEnum.PASS : RnrOrderStatusEnum.NOT_PASS;

        //获取rnrId
        RnrOrderDTO rnrOrderDTO = new RnrOrderDTO();
        rnrOrderDTO.setUuid(orderId);
        RnrOrderDTO orderDTO = orderService.getByUuid(rnrOrderDTO);

        if (null == orderDTO) {
            log.error("ManufacturerRnrCallBackHandler reviewCallBack orderDTO is null,dto = {} ", JSONObject.toJSONString(dto));
            return;
        }

        String rnrId = orderDTO.getRnrId();

        if (statusEnum == RnrOrderStatusEnum.PASS) {
            //更新orderId查询carInfo
            MgRnrCardInfoDTO mgRnrCardInfoDTO = new MgRnrCardInfoDTO();
            mgRnrCardInfoDTO.setOrderId(orderId);
            List<MgRnrCardInfoDTO> mgRnrCardInfoDTOList = cardInfoService.queryByList(mgRnrCardInfoDTO);
            String orgId = "";
            if(!CollectionUtils.isEmpty(mgRnrCardInfoDTOList)){
                OrgSimRelDTO orgSimRelDTO  =iOrgSimRelService.getByIccid(mgRnrCardInfoDTOList.get(0).getIccid());
                orgId = orgSimRelDTO.getOrgUuid();
            }
            //查询车企是否存在
            MgVehicleCompanyDTO mvc = new MgVehicleCompanyDTO();
            mvc.setTenantNo(orderDTO.getTenantNo());
            mvc.setOrgId(orgId);
            mvc =  mgVehicleCompanyService.getVehicleCompany(mvc);
            //不存在则保存
            if(mvc == null){
                mvc = new MgVehicleCompanyDTO();
                mvc.setFirstRnrId(rnrId);
                mvc.setOrgId(orderDTO.getOrgId());
                mvc.setTenantNo(orderDTO.getTenantNo());
                mvc.setCreator(orderDTO.getCreator());
                mgVehicleCompanyService.saveByRnrId(mvc);
            }

            iOrgSimRelService.updateVin(mgRnrCardInfoDTOList);
        }
        //更新数据状态
        processService.processRnrResult(orderId,rnrId,statusEnum,dto.getAuditNote());
    }
}
