package com.cusc.nirvana.user.rnr.workorder.handler.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.cusc.nirvana.common.loader.CollectionUtils;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.eiam.dto.OrganizationDTO;
import com.cusc.nirvana.user.rnr.mg.constants.CommonDeleteEnum;
import com.cusc.nirvana.user.rnr.mg.constants.CryptKeyHelper;
import com.cusc.nirvana.user.rnr.mg.constants.NoticeStatusEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrBizzTypeEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrOrderStatusEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrStatus;
import com.cusc.nirvana.user.rnr.mg.converter.MgRnrCardInfoConverter;
import com.cusc.nirvana.user.rnr.mg.dto.MgCardNoticeDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrCardInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.RnrOrderDTO;
import com.cusc.nirvana.user.rnr.mg.service.*;
import com.cusc.nirvana.user.rnr.workorder.dto.WorkOrderReviewCallBackRequestDTO;
import com.cusc.nirvana.user.rnr.workorder.handler.WorkOrderCallBackHandler;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

/**
 * 解绑工单回调
 *
 * @author yubo
 * @since 2022-04-25 16:19
 */
@Service("unbindWorkOrderCallBackHandler")
@Slf4j
public class UnbindWorkOrderCallBackHandler implements WorkOrderCallBackHandler {

    @Autowired
    IRnrOrderService orderService;

    @Autowired
    private IMgRnrInfoService rnrInfoService;

    @Autowired
    IMgRnrCardInfoService cardInfoService;

    @Autowired
    @Lazy
    private IMgRnrCardNoticeService cardNoticeService;

    @Autowired
    private IOrganService organService;




    @Override
    public void claimAssignWorkOrder(String workNo) {

    }

    @Override
    public void reviewCallBack(WorkOrderReviewCallBackRequestDTO dto) {
        String orderId = dto.getBusinessObjectId();
        RnrOrderStatusEnum statusEnum =
                StringUtils.equalsIgnoreCase(dto.getAuditResult(), "1") ? RnrOrderStatusEnum.PASS : RnrOrderStatusEnum.NOT_PASS;

        //获取rnrId
        RnrOrderDTO rnrOrderDTO = new RnrOrderDTO();
        rnrOrderDTO.setUuid(orderId);
        RnrOrderDTO orderDTO = orderService.getByUuid(rnrOrderDTO);
        rnrOrderDTO.setTenantNo(orderDTO.getTenantNo());
        if (null == orderDTO) {
            log.error("UnbindWorkOrderCallBackHandler reviewCallBack orderDTO is null,dto = {} ", JSONObject.toJSONString(dto));
            return;
        }

        String rnrId = orderDTO.getRnrId();

        //根据rnrId找到所有iccids
        List<MgRnrCardInfoDTO> cardInfoDTOS = findCardsByRnrId(rnrId);

        List<String> cardIds = new ArrayList<>();
        List<String> rnrIds = new ArrayList<>();
        List<MgCardNoticeDTO> mqList = new ArrayList<>();

        //查询车企信息
        String manufacturerOrgId;
        OrganizationDTO organ = new OrganizationDTO();
        organ.setTenantNo(orderDTO.getTenantNo());
        organ.setUuid(orderDTO.getOrgId());
        Response<OrganizationDTO> organResp = organService.getCarSubOrganByOrgId(organ);
        if (organResp != null && organResp.isSuccess() && organResp.getData() != null) {
            manufacturerOrgId = organResp.getData().getUuid();
        } else {
            log.warn("sendCardNoticeByOrderId 获取车企组织失败，{}", JSON.toJSONString(organResp));
            manufacturerOrgId = "0";
        }


        //根据iccids找到需要解绑的记录
        findUnboundInfo(cardInfoDTOS, mqList, cardIds, rnrIds, statusEnum, rnrOrderDTO, manufacturerOrgId);

        //更新状态
        saveStatus(orderId, rnrId, cardIds,rnrIds,statusEnum,dto.getAuditNote());

        //异步发送解绑消息 审核通过在发送通知
        if (CollectionUtils.isNotEmpty(mqList)&& RnrOrderStatusEnum.PASS.getCode().equals(statusEnum.getCode())) {
            log.info("异步发送消息入参{}",JSONObject.toJSONString(mqList));
            cardNoticeService.sendCardNotice(mqList);
        }
    }

    @Transactional
    public void saveStatus(String orderId, String rnrId, List<String> cardIds, List<String> rnrIds, RnrOrderStatusEnum statusEnum,String content) {
        //更新订单状态
        //orderService.updateOrderStatus(Collections.singletonList(orderId), statusEnum.getCode());
		RnrOrderDTO updateOrder = new RnrOrderDTO();
		updateOrder.setUuid(orderId);
		updateOrder.setOrderStatus(statusEnum.getCode());
		updateOrder.setAuditTime(new Date());
		updateOrder.setFinalApproveTime(new Date());
		updateOrder.setComment(content);
		orderService.update(updateOrder);
        if (statusEnum == RnrOrderStatusEnum.PASS) {
            //审核通过
            rnrInfoService.updateRnrStatus(rnrId, RnrStatus.RNR);
            //更新card状态
            cardInfoService.updateCardStatusByOrderId(orderId, NoticeStatusEnum.NEED.getCode(), RnrStatus.UNBOUND.getCode());
        } else {
            //更新rnr状态
            rnrInfoService.updateRnrStatus(rnrId, RnrStatus.RNR_FAIL);
            //更新card状态
            cardInfoService.updateCardStatusByOrderId(orderId, NoticeStatusEnum.NONEED.getCode(), RnrStatus.RNR_FAIL.getCode());
        }
        
        //原card表设置成解绑状态
		if (CollectionUtils.isNotEmpty(cardIds) && RnrOrderStatusEnum.PASS.equals(statusEnum)) {
			cardInfoService.unboundCardBatch(cardIds, RnrStatus.UNBOUND);
		}


        //原rnr表设置成解绑状态
//        rnrInfoService.updateRnrStatusBatch(rnrIds, RnrStatus.UNBOUND);
    }

    /**
     * 获取待解绑的卡信息
     *
     * @param cardInfoDTOS
     * @param mqList
     * @param cardIds
     * @param rnrIds
     */
    private void findUnboundInfo(List<MgRnrCardInfoDTO> cardInfoDTOS, List<MgCardNoticeDTO> mqList, List<String> cardIds, List<String> rnrIds, RnrOrderStatusEnum statusEnum,
                                 RnrOrderDTO orderDTO, String manufacturerOrgId) {
        for (MgRnrCardInfoDTO cardInfoDTO : cardInfoDTOS) {
            List<MgRnrCardInfoDTO> bindCards = findBindCards(cardInfoDTO.getIccid());

            for (MgRnrCardInfoDTO bindCard : bindCards) {
                cardIds.add(bindCard.getUuid());
                rnrIds.add(bindCard.getRnrId());

                MgCardNoticeDTO mgCardNoticeDTO = new MgCardNoticeDTO();
                mgCardNoticeDTO.setVin(bindCard.getIotId());
                mgCardNoticeDTO.setIccid(bindCard.getIccid());
                mgCardNoticeDTO.setRnrBizzType(RnrBizzTypeEnum.Unbound.getCode());
                mgCardNoticeDTO.setOrderId(orderDTO.getUuid());
                mgCardNoticeDTO.setTenantNo(orderDTO.getTenantNo());
                mgCardNoticeDTO.setResultCode(statusEnum.getCode());
                mgCardNoticeDTO.setManufacturerOrgId(manufacturerOrgId);
                mqList.add(mgCardNoticeDTO);
            }
        }
    }

    //根据iccid寻找已绑定的记录
    private List<MgRnrCardInfoDTO> findBindCards(String iccid) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());
        queryWrapper.eq("iccid", CryptKeyHelper.encrypt(iccid));
        queryWrapper.eq("rnr_status", RnrStatus.RNR.getCode());
        List list = cardInfoService.list(queryWrapper);
        return MgRnrCardInfoConverter.INSTANCE.poListToDtoList(list);
    }

    //根据rnrid找到iccis
    private List<MgRnrCardInfoDTO> findCardsByRnrId(String rnrId) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("is_delete", CommonDeleteEnum.NORMAL.getCode());
        queryWrapper.eq("rnr_id", rnrId);
//        queryWrapper.eq("rnr_status", RnrStatus.RNR.getCode());
        List list = cardInfoService.list(queryWrapper);
        return MgRnrCardInfoConverter.INSTANCE.poListToDtoList(list);
    }
}
